"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var os_1 = require("os");
var path_1 = require("path");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var uuid_1 = require("uuid");
var create_components_command_1 = require("../src/commands/create.components.command");
var rx_copy_1 = require("../src/utils/rx.copy");
var rx_delete_1 = require("../src/utils/rx.delete");
var expect_utils_1 = require("./utils/expect.utils");
var test_utils_1 = require("./utils/test.utils");
describe('create.components.command', function () {
    it('should work with empty types', function () {
        var dir = os_1.tmpdir();
        var name = uuid_1.v4();
        var dstBase = path_1.join(dir, name);
        var dstSrc = dstBase;
        var dstData = path_1.join(dstBase, 'data');
        var assetBase = test_utils_1.getAssetBaseDir();
        var srcSrc = path_1.join(assetBase, 'test', 'empty-project');
        var srcData = path_1.join(assetBase, 'test', 'empty-types');
        // copy the data
        var rxData = rxjs_1.merge(rx_copy_1.rxCopyDir(srcSrc, dstSrc), rx_copy_1.rxCopyDir(srcData, dstData));
        var rxDelete = rx_delete_1.rxDeleteDir(dstBase);
        var opts = {
            command: create_components_command_1.CreateComponentsCommand.COMMAND,
            commandType: create_components_command_1.CreateComponentsCommand.TYPE,
            src: path_1.join(dstSrc, 'src'),
            modules: true,
            data: dstData,
            pkgDir: dstSrc,
            verbose: true,
            override: true,
            allArgs: []
        };
        var cmd = new create_components_command_1.CreateComponentsCommand(opts);
        var rxCmd = cmd.process();
        // setup and teardown
        var rxSetup = rxjs_1.concat(rxDelete, rxData);
        var rxTeardown = rxDelete;
        // command
        var rxTest = rxCmd.pipe(operators_1.mergeMap(expect_utils_1.expectFileExists), operators_1.toArray());
        // execute the command
        var rxTotal = rxjs_1.concat(rxSetup, rxTest, rxTeardown);
        return rxTotal.toPromise();
    });
    it('should work with ryans group elements', function () {
        var dir = os_1.tmpdir();
        var name = 'ryan';
        var dstBase = path_1.join(dir, name);
        var dstSrc = dstBase;
        var dstData = path_1.join(dstBase, 'data');
        var assetBase = test_utils_1.getAssetBaseDir();
        var srcSrc = path_1.join(assetBase, 'test', 'empty-project');
        var srcData = path_1.join(assetBase, 'test', 'ryan-group');
        // copy the data
        var rxData = rxjs_1.merge(rx_copy_1.rxCopyDir(srcSrc, dstSrc), rx_copy_1.rxCopyDir(srcData, dstData));
        var rxDelete = rxjs_1.empty(); // rxDeleteDir(dstBase);
        var opts = {
            command: create_components_command_1.CreateComponentsCommand.COMMAND,
            commandType: create_components_command_1.CreateComponentsCommand.TYPE,
            src: path_1.join(dstSrc, 'src'),
            data: dstData,
            pkgDir: dstSrc,
            verbose: true,
            override: true,
            allArgs: []
        };
        var cmd = new create_components_command_1.CreateComponentsCommand(opts);
        var rxCmd = cmd.process();
        // setup and teardown
        var rxSetup = rxjs_1.concat(rxDelete, rxData);
        var rxTeardown = rxDelete;
        // command
        var rxTest = rxCmd.pipe(operators_1.mergeMap(expect_utils_1.expectFileExists), operators_1.toArray());
        // execute the command
        var rxTotal = rxjs_1.concat(rxSetup, rxTest, rxTeardown);
        return rxTotal.toPromise();
    });
    fit('should work with group elements limited to a single type', function () {
        var dir = os_1.tmpdir();
        var name = 'test'; //v4();
        var dstBase = path_1.join(dir, name);
        var dstSrc = dstBase;
        var dstData = path_1.join(dstBase, 'data');
        var assetBase = test_utils_1.getAssetBaseDir();
        var srcSrc = path_1.join(assetBase, 'test', 'empty-project');
        var srcData = path_1.join(assetBase, 'test', 'group-elements');
        // copy the data
        var rxData = rxjs_1.merge(rx_copy_1.rxCopyDir(srcSrc, dstSrc), rx_copy_1.rxCopyDir(srcData, dstData));
        var rxDelete = rxjs_1.empty(); // rxDeleteDir(dstBase);
        var opts = {
            command: create_components_command_1.CreateComponentsCommand.COMMAND,
            commandType: create_components_command_1.CreateComponentsCommand.TYPE,
            src: path_1.join(dstSrc, 'src'),
            data: dstData,
            pkgDir: dstSrc,
            verbose: true,
            override: true,
            include: 'Family',
            allArgs: []
        };
        var cmd = new create_components_command_1.CreateComponentsCommand(opts);
        var rxCmd = cmd.process();
        // setup and teardown
        var rxSetup = rxjs_1.concat(rxDelete, rxData);
        var rxTeardown = rxjs_1.empty(); //rxDelete;
        // command
        var rxTest = rxCmd.pipe(operators_1.mergeMap(expect_utils_1.expectFileExists), operators_1.toArray());
        // execute the command
        var rxTotal = rxjs_1.concat(rxSetup, rxTest, rxTeardown);
        return rxTotal.toPromise();
    });
    it('should work with group elements', function () {
        var dir = os_1.tmpdir();
        var name = uuid_1.v4();
        var dstBase = path_1.join(dir, name);
        var dstSrc = dstBase;
        var dstData = path_1.join(dstBase, 'data');
        var assetBase = test_utils_1.getAssetBaseDir();
        var srcSrc = path_1.join(assetBase, 'test', 'empty-project');
        var srcData = path_1.join(assetBase, 'test', 'group-elements');
        // copy the data
        var rxData = rxjs_1.merge(rx_copy_1.rxCopyDir(srcSrc, dstSrc), rx_copy_1.rxCopyDir(srcData, dstData));
        var rxDelete = rx_delete_1.rxDeleteDir(dstBase);
        var opts = {
            command: create_components_command_1.CreateComponentsCommand.COMMAND,
            commandType: create_components_command_1.CreateComponentsCommand.TYPE,
            src: path_1.join(dstSrc, 'src'),
            data: dstData,
            pkgDir: dstSrc,
            verbose: true,
            override: true,
            allArgs: []
        };
        var cmd = new create_components_command_1.CreateComponentsCommand(opts);
        var rxCmd = cmd.process();
        // setup and teardown
        var rxSetup = rxjs_1.concat(rxDelete, rxData);
        var rxTeardown = rxDelete;
        // command
        var rxTest = rxCmd.pipe(operators_1.mergeMap(expect_utils_1.expectFileExists), operators_1.toArray());
        // execute the command
        var rxTotal = rxjs_1.concat(rxSetup, rxTest, rxTeardown);
        return rxTotal.toPromise();
    });
});
