"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_1 = require("lodash");
var os_1 = require("os");
var path_1 = require("path");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var uuid_1 = require("uuid");
var create_components_command_1 = require("../src/commands/create.components.command");
var create_layout_command_1 = require("../src/commands/create.layout.command");
var rx_delete_1 = require("../src/utils/rx.delete");
var rx_copy_1 = require("./../src/utils/rx.copy");
var expect_utils_1 = require("./utils/expect.utils");
var test_utils_1 = require("./utils/test.utils");
describe('create.single.layout', function () {
    function findFile(aFileName, aArray) {
        return aArray.find(function (name) { return path_1.parse(name).base === aFileName; });
    }
    function findFileRegexp(aFileName, aArray) {
        return aArray.find(function (name) { return aFileName.test(name); });
    }
    it('should work with multiple layout sources', function () {
        var dir = os_1.tmpdir();
        var name = uuid_1.v4();
        var dstBase = path_1.join(dir, name);
        var assetBase = test_utils_1.getAssetBaseDir();
        var dstData = path_1.join(dir, name, 'data');
        var dstSrc = dstBase;
        var srcSrc = path_1.join(assetBase, 'test', 'empty-project');
        var srcData = path_1.join(assetBase, 'test', 'oslo');
        var rxDelete = rx_delete_1.rxDeleteDir(dstBase);
        // copy the data
        var rxData = rxjs_1.merge(rx_copy_1.rxCopyDir(srcSrc, dstSrc), rx_copy_1.rxCopyDir(srcData, dstData));
        var osloSrc = path_1.join(dstSrc, 'src', 'app', 'oslo');
        var stockholmSrc = path_1.join(dstSrc, 'src', 'app', 'stockholm');
        // create the oslo components
        var cmpOpt = {
            command: create_components_command_1.CreateComponentsCommand.COMMAND,
            commandType: create_components_command_1.CreateComponentsCommand.TYPE,
            data: dstData,
            src: osloSrc,
            verbose: false,
            override: true,
            modules: true,
            flat: true,
            skipRegistration: true,
            scss: true,
            allArgs: []
        };
        var cmpCmd = new create_components_command_1.CreateComponentsCommand(cmpOpt);
        var rxCmpCmd = cmpCmd.process();
        // add an override in a different folder
        var layoutOpt = {
            command: create_layout_command_1.CreateLayoutCommand.COMMAND,
            commandType: create_layout_command_1.CreateLayoutCommand.TYPE,
            name: 'weird name',
            type: 'Hero video',
            data: dstData,
            src: [stockholmSrc, osloSrc],
            verbose: false,
            override: true,
            skipRegistration: true,
            flat: true,
            modules: true,
            scss: true,
            allArgs: []
        };
        var layoutCmd = new create_layout_command_1.CreateLayoutCommand(layoutOpt);
        var rxLayoutCmd = layoutCmd.process();
        var rxFiles = rxjs_1.concat(rxCmpCmd, rxLayoutCmd).pipe(operators_1.distinct(), operators_1.mergeMap(expect_utils_1.expectFileExists), operators_1.map(function (file) { return lodash_1.replace(file, /\\/g, '/'); }), operators_1.toArray(), operators_1.map(function (files) {
            // make sure we have layouts
            expect(findFileRegexp(/\/app\/oslo\/layouts.ts$/g, files)).toBeDefined();
            expect(findFileRegexp(/\/app\/oslo\/layouts.exports.ts$/g, files)).toBeDefined();
            expect(findFileRegexp(/\/app\/stockholm\/layouts.ts$/g, files)).toBeDefined();
            expect(findFileRegexp(/\/app\/stockholm\/layouts.exports.ts$/g, files)).toBeDefined();
            // make sure we have modules
            expect(findFileRegexp(/\/app\/oslo\/modules.ts$/g, files)).toBeDefined();
            expect(findFileRegexp(/\/app\/oslo\/modules.exports.ts$/g, files)).toBeDefined();
            expect(findFileRegexp(/\/app\/stockholm\/modules.ts$/g, files)).toBeDefined();
            expect(findFileRegexp(/\/app\/stockholm\/modules.exports.ts$/g, files)).toBeDefined();
            // test that we have the desired layout at the correct location
            expect(findFileRegexp(/\/app\/stockholm\/layouts\/weird-name\/weirdNameLayout.ts$/g, files)).toBeDefined();
            expect(findFileRegexp(/\/app\/oslo\/layouts\/hero-video\/heroVideoLayout.ts$/g, files)).toBeDefined();
        }));
        var rxTotal = rxjs_1.concat(rxDelete, rxData, rxFiles, rxDelete);
        return rxTotal.toPromise();
    });
    it('should create a layout selecting the name automatically', function () {
        var dir = os_1.tmpdir();
        var name = uuid_1.v4();
        var dstBase = path_1.join(dir, name);
        var assetBase = test_utils_1.getAssetBaseDir();
        var dstData = path_1.join(dir, name, 'data');
        var dstSrc = dstBase;
        var srcSrc = path_1.join(assetBase, 'test', 'empty-project');
        var srcData = path_1.join(assetBase, 'test', 'oslo');
        var rxDelete = rx_delete_1.rxDeleteDir(dstBase);
        // copy the data
        var rxData = rxjs_1.merge(rx_copy_1.rxCopyDir(srcSrc, dstSrc), rx_copy_1.rxCopyDir(srcData, dstData));
        // create layouts on the result
        var layoutOpt = {
            command: create_layout_command_1.CreateLayoutCommand.COMMAND,
            commandType: create_layout_command_1.CreateLayoutCommand.TYPE,
            type: 'Hero video',
            data: dstData,
            src: path_1.join(dstSrc, 'src', 'app'),
            verbose: false,
            override: true,
            modules: true,
            skipRegistration: false,
            scss: true,
            allArgs: []
        };
        var layoutCmd = new create_layout_command_1.CreateLayoutCommand(layoutOpt);
        var rxLayoutCmd = layoutCmd.process().pipe(operators_1.mergeMap(expect_utils_1.expectFileExists), operators_1.toArray(), operators_1.map(function (files) {
            expect(findFile('layouts.ts', files)).toBeDefined();
            expect(findFile('layouts.exports.ts', files)).toBeDefined();
            // layout files
            expect(findFile('heroVideoLayout.ts', files)).toBeDefined();
            expect(findFile('heroVideoLayout.scss', files)).toBeDefined();
            expect(findFile('heroVideoLayout.html', files)).toBeDefined();
            // component files
            expect(findFile('abstractHeroVideoComponent.ts', files)).toBeDefined();
            expect(findFile('typeHeroVideoComponent.scss', files)).toBeDefined();
            expect(findFile('typeHeroVideoComponent.ts', files)).toBeDefined();
            expect(findFile('typeHeroVideoComponent.html', files)).toBeDefined();
        }));
        var rxTotal = rxjs_1.concat(rxDelete, rxData, rxLayoutCmd, rxDelete);
        return rxTotal.toPromise();
    });
    it('should create a layout given a predfined name', function () {
        var dir = os_1.tmpdir();
        var name = uuid_1.v4();
        var dstBase = path_1.join(dir, name);
        var assetBase = test_utils_1.getAssetBaseDir();
        var dstData = path_1.join(dir, name, 'data');
        var dstSrc = dstBase;
        var srcSrc = path_1.join(assetBase, 'test', 'empty-project');
        var srcData = path_1.join(assetBase, 'test', 'oslo');
        var rxDelete = rx_delete_1.rxDeleteDir(dstSrc);
        // copy the data
        var rxData = rxjs_1.merge(rx_copy_1.rxCopyDir(srcSrc, dstSrc), rx_copy_1.rxCopyDir(srcData, dstData));
        // create layouts on the result
        var layoutOpt = {
            command: create_layout_command_1.CreateLayoutCommand.COMMAND,
            commandType: create_layout_command_1.CreateLayoutCommand.TYPE,
            name: 'weird name',
            type: 'Hero video',
            data: dstData,
            src: path_1.join(dstSrc, 'src', 'app'),
            verbose: false,
            override: true,
            skipRegistration: false,
            scss: true,
            allArgs: []
        };
        var layoutCmd = new create_layout_command_1.CreateLayoutCommand(layoutOpt);
        var rxLayoutCmd = layoutCmd.process().pipe(operators_1.mergeMap(expect_utils_1.expectFileExists), operators_1.toArray(), operators_1.map(function (files) {
            expect(findFile('layouts.ts', files)).toBeDefined();
            expect(findFile('layouts.exports.ts', files)).toBeDefined();
            // layout files
            expect(findFile('heroVideoLayout.ts', files)).toBeDefined();
            expect(findFile('heroVideoLayout.scss', files)).toBeDefined();
            expect(findFile('heroVideoLayout.html', files)).toBeDefined();
            // layout files
            expect(findFile('weirdNameLayout.ts', files)).toBeDefined();
            expect(findFile('weirdNameLayout.scss', files)).toBeDefined();
            expect(findFile('weirdNameLayout.html', files)).toBeDefined();
            // component files
            expect(findFile('abstractHeroVideoComponent.ts', files)).toBeDefined();
            expect(findFile('typeHeroVideoComponent.scss', files)).toBeDefined();
            expect(findFile('typeHeroVideoComponent.ts', files)).toBeDefined();
            expect(findFile('typeHeroVideoComponent.html', files)).toBeDefined();
        }));
        var rxTotal = rxjs_1.concat(rxDelete, rxData, rxLayoutCmd, rxDelete);
        return rxTotal.toPromise();
    });
});
