"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
/* Copyright IBM Corp. 2017 */
var path_1 = require("path");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var url_1 = require("url");
var file_utils_1 = require("../utils/file.utils");
var constants_1 = require("./../constants");
var options_1 = require("./../options");
var config_1 = require("./../utils/config");
var hash_utils_1 = require("./../utils/hash.utils");
var rx_hbs_1 = require("./../utils/rx.hbs");
var rx_walk_1 = require("./../utils/rx.walk");
var abstract_command_1 = require("./abstract.command");
var help_1 = require("./help");
var _DEFAULT_SITE = 'default';
var JSONP_CALLBACK = 'callback=JSONP_CALLBACK';
// name of the SDK module
var _MODULE_NAME = 'WchSdk';
var _JSONP_CALLBACK_PREFIX = _MODULE_NAME + ".jsonp.callback.";
var RENDERING_PREFIX = 'delivery/v1/rendering/';
var CONTEXT_PREFIX = 'delivery/v1/rendering/context/';
var SITES_PREFIX = 'delivery/v1/rendering/sites/';
function _getSiteUrl(aSite) {
    var siteID = aSite.data.id || _DEFAULT_SITE;
    // replace
    return "" + SITES_PREFIX + encodeURIComponent(siteID);
}
/**
 * Returns the URI path for the rendering context
 *
 * @param aID   the ID
 * @return the path
 */
function _getRenderingContextURL(aID) {
    return "" + CONTEXT_PREFIX + encodeURIComponent(aID);
}
function _getHash(aToken) {
    return hash_utils_1.hashToIdentifier(hash_utils_1.hashString(hash_utils_1.longHash(), aToken));
}
function _getPreFetchURL(aUrl) {
    // append callback
    var url = aUrl;
    if (url.indexOf('?') >= 0) {
        url += '&' + JSONP_CALLBACK;
    }
    else {
        url += '?' + JSONP_CALLBACK;
    }
    // construct the digest
    var digest = _getHash(url);
    // replace
    return url.replace(/=JSONP_CALLBACK(&|$)/, function (match, suffix) { return "=" + encodeURIComponent(_JSONP_CALLBACK_PREFIX + digest) + suffix; });
}
function _forEachContent(aSite, aLevel, aFilter) {
    // check if we have a content item
    var contentId = (aSite && aSite.data && aSite.data.contentId) ? aSite.data.contentId : undefined;
    var data = (aSite && aSite.data) ? aSite.data : undefined;
    var obContentId = (contentId && aFilter(aLevel, aSite)) ? rxjs_1.of({
        id: contentId,
        data: data,
        level: aLevel
    }) : rxjs_1.empty();
    // children
    var children = (aSite && aSite.children) ? aSite.children : [];
    var obChildren = rxjs_1.from(children).pipe(operators_1.mergeMap(function (child) { return _forEachContent(child, aLevel + 1, aFilter); }));
    // merge
    return rxjs_1.merge(obContentId, obChildren);
}
function _sortByNumber(aLeft, aRight) {
    return (aLeft < aRight) ? -1 : (aLeft > aRight) ? +1 : 0;
}
function _sortLevels(aLeft, aRight) {
    // compare by level, then by position, then by name
    var cmp = _sortByNumber(aLeft.level, aRight.level);
    if (cmp === 0) {
        // by position
        var lPos = aLeft.data ? aLeft.data.position : 0;
        var rPos = aRight.data ? aRight.data.position : 0;
        cmp = _sortByNumber(lPos, rPos);
        if (cmp === 0) {
            // sort by ID
            return aLeft.id.localeCompare(aRight.id);
        }
    }
    // ok
    return cmp;
}
function _createUrl(aApiBase, aRelUrl, bRelative) {
    // parse
    var url = "" + aApiBase + aRelUrl;
    var parsed = url_1.parse(url);
    return (bRelative ? parsed.path : "//" + parsed.host + parsed.path) || url;
}
function _getRenderingSuffix(aUrl) {
    var idx = aUrl.indexOf(RENDERING_PREFIX);
    return aUrl.substr(idx);
}
function _getPreFetchContent(aApiBase, aSite, aFilter, aLogger) {
    // only continue if we have a site
    return _getPreFetchURLs(aApiBase, aSite, aFilter, aLogger, false).pipe(
    // strip the suffix
    operators_1.map(_getRenderingSuffix));
}
/**
 * Generates the prefetch URLs
 *
 * @param aApiBase  the API base URL
 * @param aSite     the site
 * @param aFilter
 * @param aLogger
 * @param bRelative
 */
function _getPreFetchURLs(aApiBase, aSite, aFilter, aLogger, bRelative) {
    // log this
    aLogger.next("API base [" + aApiBase + "].");
    // only continue if we have a site
    if (aSite) {
        // site URL
        var siteURL = _getSiteUrl(aSite);
        var sitePreFetchURL = _getPreFetchURL(siteURL);
        var obSitePreFetchURL = rxjs_1.of(sitePreFetchURL);
        // generate a list
        var obChildren = _forEachContent(aSite, 0, aFilter).pipe(operators_1.toArray(), operators_1.map(function (ch) { return ch.sort(_sortLevels); }), operators_1.tap(function (levels) { return aLogger.next("Levels [" + JSON.stringify(levels, undefined, 2) + "]."); }), operators_1.mergeMap(function (ch) { return rxjs_1.from(ch); }), operators_1.map(function (l) { return l.id; }), operators_1.map(function (id) { return _getRenderingContextURL(id); }), operators_1.map(_getPreFetchURL));
        // ok
        return rxjs_1.merge(obSitePreFetchURL, obChildren).pipe(operators_1.map(function (relURL) { return _createUrl(aApiBase, relURL, bRelative); }));
    }
    else {
        // log this
        aLogger.next('Site not available, skipping generation of preview URLs.');
        // nothing
        return rxjs_1.empty();
    }
}
function _getPreFetchTemplate() {
    return rx_hbs_1.hbsCompile(path_1.join(__dirname, '..', '..', '..', 'assets', 'prefetch.hbs'));
}
function _getPreFetchScriptTemplate() {
    return rx_hbs_1.hbsCompile(path_1.join(__dirname, '..', '..', '..', 'assets', 'prefetch.script.hbs'));
}
function _getPreFetchScript() {
    return rx_walk_1.readFile(path_1.join(__dirname, '..', '..', '..', 'assets', 'prefetch/add.prefetch.links.min.js'));
}
function _getPreFetchScriptMarkup(aContent) {
    // returns the markup
    var rxTemplate = _getPreFetchScriptTemplate();
    var rxScript = _getPreFetchScript();
    // build the context
    var rxContent = aContent.pipe(operators_1.toArray(), operators_1.map(function (data) { return JSON.stringify(data); }));
    var rxContext = rxjs_1.combineLatest(rxScript, rxContent).pipe(operators_1.map(function (_a) {
        var aScript = _a[0], aContentLinks = _a[1];
        return ({
            SCRIPT: aScript,
            CONTENT: aContentLinks
        });
    }));
    // apply the template
    // convert
    return rxjs_1.combineLatest(rxTemplate, rxContext).pipe(operators_1.map(function (_a) {
        var tmp = _a[0], ctx = _a[1];
        return tmp(ctx);
    }));
}
function _createPreFetchScriptMarkup(aApiBase, aSite, aFilter, aLogger, bRelative) {
    // get the links
    var rxContent = _getPreFetchContent(aApiBase, aSite, aFilter, aLogger);
    // combine
    return _getPreFetchScriptMarkup(rxContent);
}
function _getPreFetchMarkup(aLinks) {
    // returns the markup
    var rxTemplate = _getPreFetchTemplate();
    // convert all string into an array
    var rxContext = aLinks.pipe(operators_1.toArray());
    // convert
    return rxjs_1.combineLatest(rxTemplate, rxContext).pipe(operators_1.map(function (_a) {
        var tmp = _a[0], ctx = _a[1];
        return tmp(ctx);
    }));
}
function _allNodes(aLevel, aNode) {
    return true;
}
var REPLACE_REGEXP = /<!--\s+Prefetch begin\s+-->((.|\n|\r)*)<!--\s+Prefetch end\s+-->/g;
var BASE_REGEXP = /<base\s+[^>]*>/g;
function _insertMarkup(aPreRender, aMarkup) {
    // check if we have an expression
    if (REPLACE_REGEXP.test(aMarkup)) {
        // replace
        return aMarkup.replace(REPLACE_REGEXP, function () { return aPreRender.trim(); });
    }
    else {
        // insert after the base tag
        return aMarkup.replace(BASE_REGEXP, function (m, o) { return m + '\n' + aPreRender.trim(); });
    }
}
function _rewriteIndex(aMarkup, aSrcDir) {
    // read the index file
    var fileName = path_1.join(aSrcDir, 'index.html');
    return rx_walk_1.readFile(fileName).pipe(operators_1.map(function (content) { return _insertMarkup(aMarkup, content); }), operators_1.mergeMap(function (newMarkup) { return rx_walk_1.writeFile(fileName, newMarkup); }));
}
/**
 * Generates layout and layout mappings
 *
 * add prefetch --data DATA_FOLDER --src SRC_FOLDER
 */
var AddPreloadCommand = /** @class */ (function (_super) {
    __extends(AddPreloadCommand, _super);
    function AddPreloadCommand(options) {
        var _this = _super.call(this, options) || this;
        _this.options = options;
        return _this;
    }
    AddPreloadCommand.prototype.printHelp = function () {
        // log the command line
        console.log(AddPreloadCommand.COMMAND, AddPreloadCommand.TYPE, "--" + constants_1.OPTION_SOURCE + " <SOURCE>", "--" + constants_1.OPTION_DATA + " <DATA>", "[--" + constants_1.OPTION_NAME + " <NAME>]", "[--" + constants_1.OPTION_ROOT + " <ROOT>]", "[--" + constants_1.OPTION_RELATIVE + "]", "[--" + constants_1.OPTION_SCRIPT + "]", "[--" + constants_1.OPTION_VERBOSE + "]");
        // log the parameters
        console.log("--" + constants_1.OPTION_SOURCE + " <SOURCE>", ':', help_1.HELP_COPY_SOURCE);
        console.log("--" + constants_1.OPTION_DATA + " <DATA>", ':', help_1.HELP_DATA_DIRECTORY);
        console.log("--" + constants_1.OPTION_NAME + " <NAME>", ':', help_1.HELP_APPLICATION_NAME);
        console.log("--" + constants_1.OPTION_ROOT, ':', help_1.HELP_CONTEXT_ROOT);
        console.log("--" + constants_1.OPTION_RELATIVE, ':', help_1.HELP_RELATIVE);
        console.log("--" + constants_1.OPTION_VERBOSE, ':', help_1.HELP_VERBOSE);
    };
    AddPreloadCommand.prototype.process = function () {
        // options
        var opts = this.options;
        // the directories
        var dataDirOpt = this.getDataDir();
        var appName = opts.name || this.getAppName();
        var contextRoot = "/" + config_1.configTrimSlashes(opts.root || this.getContextRoot(appName));
        // the directories
        var srcDirOpt = options_1.optNormalizeDir(file_utils_1.firstOf(opts.src) || config_1.configIndexDirFromContextRoot(contextRoot, dataDirOpt));
        var srcDir = this.rxMkdirp(srcDirOpt);
        // base URL
        var apiBase = this.getApiBase();
        // relative or absolute
        var bRelative = !!opts.relative;
        // filter
        var filter = _allNodes;
        // the logger
        var logger = this.getLogger();
        // log the settings
        logger.next("Data dir [" + dataDirOpt + "], source dir [" + srcDirOpt + "], relative [" + bRelative + "].");
        // tools
        var rxWchTools = this.getWchTools();
        // check the script case
        var bScript = !!opts.script;
        // get the markup
        var rxMarkup = rxjs_1.combineLatest(rxWchTools, apiBase).pipe(operators_1.mergeMap(function (_a) {
            var aWchTools = _a[0], aApiBase = _a[1];
            return bScript ? _createPreFetchScriptMarkup(aApiBase, aWchTools.sites ? aWchTools.sites['default'] : undefined, filter, logger, bRelative) :
                _getPreFetchMarkup(_getPreFetchURLs(aApiBase, aWchTools.sites ? aWchTools.sites['default'] : undefined, filter, logger, bRelative));
        }));
        // rewrite the index
        var rxIndex = rxjs_1.combineLatest(srcDir, rxMarkup).pipe(operators_1.mergeMap(function (_a) {
            var aSrcDir = _a[0], aMarkup = _a[1];
            return _rewriteIndex(aMarkup, aSrcDir);
        }));
        // done
        return rxIndex;
    };
    AddPreloadCommand.TYPE = 'prefetch';
    AddPreloadCommand.COMMAND = constants_1.CMD_ADD;
    return AddPreloadCommand;
}(abstract_command_1.AbstractCommand));
exports.AddPreloadCommand = AddPreloadCommand;
