"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
/* Copyright IBM Corp. 2018 */
var path_1 = require("path");
var process_1 = require("process");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var url_1 = require("url");
var constants_1 = require("../constants");
var assert_1 = require("../utils/assert");
var file_utils_1 = require("../utils/file.utils");
var json_1 = require("../utils/json");
var rx_delete_1 = require("../utils/rx.delete");
var rx_file_1 = require("../utils/rx.file");
var rx_walk_1 = require("../utils/rx.walk");
var rx_zip_1 = require("../utils/rx.zip");
var options_1 = require("./../options");
var tenant_1 = require("./../utils/tenant");
var transliteration_1 = require("./../utils/transliteration");
var abstract_command_1 = require("./abstract.command");
var create_project_command_1 = require("./create.project.command");
// relative path to the data directory
var REL_DATA_DIR = './src/wchLayouts';
var REL_SOURCE_DIR = './src';
var DOMAIN_NAME_REGEXP = /(?:\/\/\s*)(.+DOMAIN_NAME\s+=\s+['"])[^'"]+(["'];)/;
var HUB_ID_REGEXP = /(?:\/\/\s*)(.+CONTENT_HUB_ID\s+=\s+['"])[^'"]+(["'];)/;
function _rewriteFiles(aName, aWchToolsOptions, aLogger) {
    function _updateConstants(aData) {
        // parse the url
        var parsedUrl = url_1.parse(aWchToolsOptions.baseUrl);
        // replace the domain name
        var text = aData;
        if (parsedUrl.host) {
            text = text.replace(DOMAIN_NAME_REGEXP, function (total, prefix, suffix) { return "" + prefix + parsedUrl.host + suffix; });
        }
        // tenant ID
        if (parsedUrl.pathname) {
            var result = tenant_1.TENANT_BASED_URL_REGEXP.exec(parsedUrl.pathname);
            if (result && (result.length > 1)) {
                // extract the tenant
                var tenantID_1 = result[1];
                text = text.replace(HUB_ID_REGEXP, function (total, prefix, suffix) { return "" + prefix + tenantID_1 + suffix; });
            }
        }
        // ok
        return text;
    }
    function _rewriteConstants(aFile) {
        return rx_file_1.rxReadTextFile(aFile).pipe(operators_1.map(function (data) { return _updateConstants(data); }), operators_1.mergeMap(function (data) { return rx_file_1.rxWriteTextFile(aFile, data); }));
    }
    function _rewriteTypings(aFile) {
        return create_project_command_1.createProjectUpdateTypings(aFile, aLogger);
    }
    function _updatePackage(aPackage) {
        // add a config section
        var cfg = aPackage.config || (aPackage.config = {});
        // update
        cfg.root = '/';
        cfg.data = REL_DATA_DIR;
        cfg.src = REL_SOURCE_DIR;
        cfg.name = aName;
        // update the application name
        aPackage.name = aName;
        return aPackage;
    }
    function _rewritePackageJson(aFile) {
        return rx_walk_1.readFile(aFile).pipe(operators_1.map(function (data) { return JSON.parse(data); }), operators_1.map(function (pkg) { return _updatePackage(pkg); }), operators_1.map(json_1.canonicalizeJSON), operators_1.map(function (json) { return JSON.stringify(json, null, 2); }), operators_1.mergeMap(function (pkg) { return rx_walk_1.writeFile(aFile, pkg); }));
    }
    function _rewriteFile(aFile) {
        // parse
        var parsed = path_1.parse(aFile);
        // test some special cases
        switch (parsed.base) {
            case 'package.json':
                return _rewritePackageJson(aFile);
            case 'typings.d.ts':
                return _rewriteTypings(aFile);
            case 'Constants.ts':
                return _rewriteConstants(aFile);
            default:
                return rxjs_1.of(aFile);
        }
    }
    // the rewrite operator
    return function (aSrc) {
        // returns the rewrite operators
        return aSrc.pipe(operators_1.mergeMap(_rewriteFile));
    };
}
function _extractFromUrl(aSrcUrl, aDstDir, aLogger) {
    // log this
    aLogger.next("Extracting [" + aSrcUrl + "] to [" + aDstDir + "].");
    // just dispatch to the magic extractor, skip the first target dir
    return rx_zip_1.rxUnzipFromUrl(aSrcUrl, aDstDir, 1);
}
var CreateFromArchiveCommand = /** @class */ (function (_super) {
    __extends(CreateFromArchiveCommand, _super);
    function CreateFromArchiveCommand(options) {
        var _this = _super.call(this, options) || this;
        _this.options = options;
        return _this;
    }
    CreateFromArchiveCommand.prototype.process = function () {
        // options
        var opts = this.options;
        // archive
        var archive = opts.archive;
        assert_1.assertParameter(archive, constants_1.OPTION_ARCHIVE);
        assert_1.assertIsUrl(archive);
        // switch to the target directory
        var srcDir = options_1.optNormalizeDir(file_utils_1.firstOf(opts.src) || '.');
        var rxSrcDir = this.rxMkdirp(srcDir);
        // capture the current working directory
        var currentDir = process_1.cwd();
        var name = transliteration_1.kebabCase(opts.name || this.getAppName());
        var dstDir = path_1.join(srcDir, name);
        // pull content?
        var bSkipPull = opts.skipPull || false;
        // check if we would override
        var rxClean = (this.options.override && name)
            ? rxSrcDir.pipe(operators_1.mergeMap(function (aSrcDir) { return rx_delete_1.rxDeleteDir(dstDir).pipe(operators_1.mergeMapTo(rxSrcDir)); }))
            : rxSrcDir;
        // data directory
        var dataDir = path_1.normalize(path_1.join(dstDir, REL_DATA_DIR));
        // logger
        var logger = this.getLogger();
        // base URL
        var rxToolsOpts = this.getWchToolsOptions().pipe(operators_1.shareReplay(1));
        // check if we create an inline edit experience
        var bEditable = opts.editable || false;
        // write the wchtools config
        var rxCreateWchtoolsOptions = rxToolsOpts.pipe(operators_1.mergeMap(function (cfg) { return create_project_command_1.createProjectCreateWchtoolsOptions(dataDir, cfg, logger); }));
        // copy some standard resources
        var rxCopyResources = rxToolsOpts.pipe(operators_1.mergeMap(function (cfg) { return create_project_command_1.createProjectCopyResources(srcDir, name, cfg, logger); }));
        var rxRewriteOperator = rxToolsOpts.pipe(operators_1.map(function (cfg) { return _rewriteFiles(name, cfg, logger); }));
        // extract the files
        var rxExtract = rxClean.pipe(operators_1.mergeMap(function () { return _extractFromUrl(archive || '', dstDir, logger); }));
        var rxExtractAndRewrite = rxRewriteOperator.pipe(operators_1.mergeMap(function (op) { return rxExtract.pipe(op); }));
        var rxAfterExtract = rxjs_1.merge(rxCreateWchtoolsOptions, rxCopyResources);
        // total
        var rxTotal = rxjs_1.concat(rxExtractAndRewrite, rxAfterExtract).pipe(operators_1.distinct());
        // construct the target
        return rxTotal;
    };
    CreateFromArchiveCommand.TYPE = 'fromarchive';
    CreateFromArchiveCommand.COMMAND = constants_1.CMD_CREATE;
    return CreateFromArchiveCommand;
}(abstract_command_1.AbstractCommand));
exports.CreateFromArchiveCommand = CreateFromArchiveCommand;
