"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
/* Copyright IBM Corp. 2017 */
var lodash_1 = require("lodash");
var mime_1 = require("mime");
var path_1 = require("path");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var uuid_1 = require("uuid");
var constants_1 = require("./../constants");
var json_1 = require("./../utils/json");
var revision_utils_1 = require("./../utils/revision.utils");
var rx_walk_1 = require("./../utils/rx.walk");
var transliteration_1 = require("./../utils/transliteration");
var abstract_command_1 = require("./abstract.command");
var create_asset_command_1 = require("./create.asset.command");
var create_layout_command_1 = require("./create.layout.command");
var help_1 = require("./help");
function _fixPageType(aType, aName, aThumbId, aTools) {
    // construct the id
    var id = _getType(aName, aTools) || uuid_1.v4();
    // add ID
    aType.id = id;
    aType.name = aName;
    // type name
    aType.thumbnail = {
        id: aThumbId
    };
    // inject the revision field
    return json_1.canonicalizeJSON(revision_utils_1.addRevisionId(aType));
}
function _createPageTypeDoc(aName, aThumbId, aTools) {
    // load the template
    return rx_walk_1.readFile(path_1.join(__dirname, '..', '..', '..', 'assets', 'page.type.json')).pipe(operators_1.map(function (data) { return JSON.parse(data); }), operators_1.map(function (template) { return _fixPageType(template, aName, aThumbId, aTools); }));
}
function _getType(aName, aTools) {
    // iterate
    // tslint:disable-next-line:forin
    for (var id in aTools.types) {
        // content type
        var doc = aTools.types[id];
        if (doc && doc.name === aName) {
            return id;
        }
    }
}
function _hasType(aName, aTools) {
    return !!_getType(aName, aTools);
}
function _writeThumbnail(aName, aDataDir, aOpts, aLogger) {
    // id
    var id = transliteration_1.kebabCase(aName);
    // target image
    var srcThumb = path_1.join(__dirname, '..', '..', '..', 'assets', 'page.thumbnail.jpg');
    // construct an asset for the thumbnail
    return rxjs_1.of({
        command: create_asset_command_1.CreateAssetCommand.COMMAND,
        commandType: create_asset_command_1.CreateAssetCommand.TYPE,
        data: aDataDir,
        src: srcThumb,
        name: id + ".jpg",
        verbose: aOpts.verbose,
        allArgs: aOpts.allArgs
    }).pipe(operators_1.map(function (opts) { return new create_asset_command_1.CreateAssetCommand(opts); }), operators_1.mergeMap(function (cmd) { return cmd.process(); }));
}
function _isJSON(aName) {
    return !!(aName && (mime_1.getType(aName) === 'application/json'));
}
function _getThumbId(aFiles) {
    // filter down to JSON files
    return aFiles.pipe(operators_1.filter(_isJSON), operators_1.filter(function (name) { return (name.indexOf('_amd') >= 0); }), operators_1.mergeMap(function (file) { return rx_walk_1.readFile(file); }), operators_1.map(function (data) { return JSON.parse(data); }), operators_1.map(function (data) { return data.id; }));
}
/**
 * Produces a type document in the data directory
 *
 * @param aName     the name of the type
 * @param aDataDir  the data directory
 */
function _writePageType(aName, aDataDir, aTools, aOverride, aOptions, aLogger) {
    // check if we know the type
    if (!aOverride && _hasType(aName, aTools)) {
        return rxjs_1.empty();
    }
    // construct the target directory
    var dstDir = path_1.join(aDataDir, 'types');
    // thumb
    var thumb = _writeThumbnail(aName, aDataDir, aOptions, aLogger).pipe(operators_1.shareReplay(1));
    // find the ID of the generated thumb
    var thumbId = _getThumbId(thumb);
    // produce
    var json = rx_walk_1.rxMkdirp(dstDir).pipe(operators_1.mergeMap(function (dir) {
        return thumbId.pipe(operators_1.mergeMap(function (thId) { return _createPageTypeDoc(aName, thId, aTools).pipe(operators_1.mergeMap(function (doc) { return rx_walk_1.writeFile(path_1.join(dir, doc.id + "_tmd.json"), JSON.stringify(doc, undefined, 2)); })); }));
    }));
    // merge
    return rxjs_1.merge(thumb, json);
}
function _createLayout(aName, aOpt, aLogger) {
    // log this
    aLogger.next('Dispatching to CreateLayoutCommand');
    // operate
    return rxjs_1.of({
        data: aOpt.data,
        src: aOpt.src,
        type: aName,
        verbose: aOpt.verbose,
        flat: aOpt.flat,
        scss: aOpt.scss,
        module: aOpt.module,
        skipRegistration: aOpt.skipRegistration,
        command: create_layout_command_1.CreateLayoutCommand.COMMAND,
        commandType: create_layout_command_1.CreateLayoutCommand.TYPE,
        allArgs: aOpt.allArgs
    }).pipe(operators_1.map(function (opts) { return new create_layout_command_1.CreateLayoutCommand(opts); }), operators_1.mergeMap(function (cmd) { return cmd.process(); }));
}
var SUFFIX = 'type';
function _fixTypeName(aName) {
    // check for the lower case version
    var name = lodash_1.toLower(aName);
    if (name.endsWith(SUFFIX)) {
        name = aName.substring(0, aName.length - SUFFIX.length);
    }
    // append the suffix
    name = name + " " + SUFFIX;
    // camel case the beast
    return lodash_1.upperFirst(lodash_1.camelCase(name));
}
/**
 * create pagetype --data <DATA> --src <SRC> --type <TYPE> [--verbose] [--flat] [--scss] [--override]
 */
var CreatePageTypeCommand = /** @class */ (function (_super) {
    __extends(CreatePageTypeCommand, _super);
    function CreatePageTypeCommand(options) {
        var _this = _super.call(this, options) || this;
        _this.options = options;
        return _this;
    }
    CreatePageTypeCommand.prototype.printHelp = function () {
        // log the command line
        console.log(CreatePageTypeCommand.COMMAND, CreatePageTypeCommand.TYPE, "--" + constants_1.OPTION_SOURCE + " <SOURCE>", "--" + constants_1.OPTION_DATA + " <DATA>", "--" + constants_1.OPTION_TYPE + " <TYPE>", "[--" + constants_1.OPTION_FLAT + "]", "[--" + constants_1.OPTION_SCSS + "]", "[--" + constants_1.OPTION_OVERRIDE + "]", "[--" + constants_1.OPTION_VERBOSE + "]");
        // log the parameters
        console.log("--" + constants_1.OPTION_SOURCE + " <SOURCE>", ':', help_1.HELP_BUILD_SOURCE);
        console.log("--" + constants_1.OPTION_DATA + " <DATA>", ':', help_1.HELP_DATA_DIRECTORY);
        console.log("--" + constants_1.OPTION_TYPE + " <TYPE>", ':', help_1.HELP_LAYOUT_TYPE);
        console.log("--" + constants_1.OPTION_FLAT, ':', help_1.HELP_FLAT);
        console.log("--" + constants_1.OPTION_SCSS, ':', help_1.HELP_SCSS);
        console.log("--" + constants_1.OPTION_OVERRIDE, ':', help_1.HELP_OVERRIDE);
        console.log("--" + constants_1.OPTION_VERBOSE, ':', help_1.HELP_VERBOSE);
    };
    CreatePageTypeCommand.prototype.process = function () {
        // access to the data folders
        var opts = this.options;
        // the directories
        var dataDirOpt = this.getDataDir();
        var rxDataDir = this.rxMkdirp(dataDirOpt);
        // logger
        var logger = this.getLogger();
        // override flag
        var override = !!opts.override;
        // the type
        var type = _fixTypeName(opts.type || 'PageType');
        // tools
        var rxWchTools = this.getWchTools();
        // log this
        logger.next("Data dir [" + dataDirOpt + "], type [" + type + "].");
        // done
        return rxDataDir.pipe(operators_1.mergeMap(function (aDataDir) { return rxWchTools.pipe(operators_1.mergeMap(function (aWchTools) { return rxjs_1.concat(_writePageType(type, aDataDir, aWchTools, override, opts, logger), _createLayout(type, opts, logger)); })); }));
    };
    CreatePageTypeCommand.TYPE = 'pagetype';
    CreatePageTypeCommand.COMMAND = constants_1.CMD_CREATE;
    return CreatePageTypeCommand;
}(abstract_command_1.AbstractCommand));
exports.CreatePageTypeCommand = CreatePageTypeCommand;
