"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* Copyright IBM Corp. 2017 */
var graceful_fs_1 = require("graceful-fs");
var mime_1 = require("mime");
var rxjs_1 = require("rxjs");
var verror_1 = require("verror");
function _readJSON(aDesc) {
    return rxjs_1.Observable.create(function (observer) {
        // read the file
        graceful_fs_1.readFile(aDesc.absPath, 'utf-8', function (err, data) {
            if (err) {
                observer.error(err);
            }
            else {
                try {
                    observer.next({ desc: aDesc, data: JSON.parse(data) });
                    observer.complete();
                }
                catch (error) {
                    observer.error(new verror_1.VError(error, 'Unable to parse JSON file %s.', aDesc.absPath));
                }
            }
        });
    });
}
exports.readJSON = _readJSON;
function _writeJSON(aDesc) {
    return rxjs_1.Observable.create(function (observer) {
        try {
            // read the file
            graceful_fs_1.writeFile(aDesc.desc.absPath, JSON.stringify(aDesc.data, undefined, 2), function (err) {
                if (err) {
                    observer.error(err);
                }
                else {
                    observer.next(aDesc.desc);
                    observer.complete();
                }
            });
        }
        catch (error) {
            observer.error(new verror_1.VError(error, 'Unable to write JSON file %s.', aDesc.desc.absPath));
        }
    });
}
exports.writeJSON = _writeJSON;
function _isJSON(aDesc) {
    return mime_1.getType(aDesc.absPath) === 'application/json';
}
exports.isJSON = _isJSON;
