import { empty, Observable, Subject } from 'rxjs';

import { CMD_ADD } from './../constants';
import { Options } from './../options';
import { WchTools } from './../utils/wchtools';
import { AbstractCommand } from './abstract.command';
import { Command } from './command';
import { UUID_MANIFEST_TYPE } from './create.webappmanifest.command';

/* Copyright IBM Corp. 2017 */
const FAVICON_REGEXP = /<link\s+[^>]*rel\s*=\s*\"icon\"[^>]*>[\s|\r|\n]+/g;
const HEAD_REGEXP = /<\/head>/g;

function _getWebAppType(aWchTools: WchTools): any {
    return aWchTools.types[UUID_MANIFEST_TYPE];
}

function _addFavicon(aHtml: string, aType: any, aWchTools: WchTools, aLogger: Subject<string>): string {
    // the icons
    const icons: string[] = [];
    // check the image profile
    const elements: any[] = aType.elements;
    const imageElement = elements.find(el => (el.elementType === 'image') && (el.key === 'icon'));
    if (imageElement) {
        // get the profile ID
        const profileId = imageElement.imageProfileId;
        if (profileId) {
            // get the profile
            const profile = aWchTools.imageProfiles[profileId];
            if (profile) {
                // iterate over the dimensions
                profile.dimensions.forEach((d: any) => {
                    // construct the icon text
                    icons.push(`<!-- ${d.key} -->\n<link rel="icon" type="image/png" href="..." sizes="${d.width}x${d.height}">`);
                });
            } else {
                console.error(`Unable to locate the image profile for ID [${profileId}].`);
            }
        } else {
            console.error(`No profile ID for image element.`);
        }
    } else {
        console.error(`Unable to locate an image element for type [${aType.id}].`);
    }
    // remove the existing favicons
    let html = aHtml;
    if (icons.length > 0) {
        // replace the old icon
        html = html.replace(FAVICON_REGEXP, '');
        // add the new icons
        const iconsMarkup = icons.join('\n');
        html = html.replace(HEAD_REGEXP, `${iconsMarkup}\n</head>`);
    }
    // the rewritten markup
    return html;
}

/**
 * Generates layout and layout mappings
 *
 * add prefetch --data DATA_FOLDER --src SRC_FOLDER
 */
export class AddFaviconCommand extends AbstractCommand implements Command {

    static readonly TYPE = 'favicon';

    static readonly COMMAND = CMD_ADD;

    constructor(private options: Options) {
        super(options);
    }

    process(): Observable<string> {
        // options
        const opts = this.options;

        return empty();
    }
}
