"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
/* Copyright IBM Corp. 2017 */
var lodash_1 = require("lodash");
var mime_1 = require("mime");
var operators_1 = require("rxjs/operators");
var util_1 = require("util");
var json_1 = require("../utils/json");
var revision_utils_1 = require("../utils/revision.utils");
var rx_json_1 = require("./../utils/rx.json");
var rx_walk_1 = require("./../utils/rx.walk");
var abstract_command_1 = require("./abstract.command");
var _isArray = Array.isArray;
var _keys = Object.keys;
function _delete(aDoc, aKey, aParent) {
    delete aParent[aKey];
}
function _cleanupThumbnail(aThumb, aKey, aParent) {
    // ID is sufficient
    if (lodash_1.isString(aThumb.id)) {
        delete aThumb.path;
    }
    delete aThumb.url;
}
function _cleanupElement(aElement, aKey, aParent) {
}
function _cleanupElements(aElement, aKey, aParent) {
}
function _deleteName(aType, aKey, aParent) {
    if (aType && lodash_1.isString(aType.id)) {
        delete aType.name;
    }
}
function _cleanupMapping(aData, aKey, aParent) {
    _deleteName(aData.defaultLayout, 'defaultLayout', aData);
    var layouts = aData.layouts;
    if (util_1.isArray(layouts)) {
        layouts.forEach(_deleteName);
    }
}
function _cleanupMappings(aData, aKey, aParent) {
    if (util_1.isArray(aData)) {
        aData.forEach(_cleanupMapping);
    }
}
var _GENERIC_HANDLERS = {
    'rev': _delete,
    'publishing': _delete,
    'creatorId': _delete,
    'lastModifierId': _delete,
    'created': _delete,
    'lastModified': _delete,
    'creator': _delete,
    'lastModifier': _delete,
    'links': _delete,
    'cognitive': _delete
};
var _TYPE_HANDLERS = Object.assign({
    'thumbnail': _cleanupThumbnail,
    'elements': _cleanupElements
}, _GENERIC_HANDLERS);
var _LAYOUT_HANDLERS = Object.assign({
    'thumbnail': _cleanupThumbnail
}, _GENERIC_HANDLERS);
var _LAYOUT_MAPPING_HANDLERS = Object.assign({
    'type': _deleteName,
    'mappings': _cleanupMappings
}, _GENERIC_HANDLERS);
var _SITE_HANDLERS = Object.assign({}, _GENERIC_HANDLERS);
var _PAGE_HANDLERS = Object.assign({
// 'contentTypeId': _delete
}, _GENERIC_HANDLERS);
var _CONTENT_HANDLERS = Object.assign({
    'thumbnail': _cleanupThumbnail
}, _GENERIC_HANDLERS);
var _DELETE_TYPE = ['rev', 'creatorId', 'lastModifierId', 'created', 'lastModified', 'creator', 'lastModifier', 'links', 'cognitive'];
function _cleanupAsset(aAsset, aHandlerMap) {
    _keys(aAsset).filter(function (k) { return !!aHandlerMap[k]; }).forEach(function (k) { return aHandlerMap[k](aAsset[k], k, aAsset); });
}
function _cleanupType(aType) {
    _cleanupAsset(aType, _TYPE_HANDLERS);
}
function _cleanupLayout(aType) {
    _cleanupAsset(aType, _LAYOUT_HANDLERS);
}
function _cleanupLayoutMappings(aType) {
    _cleanupAsset(aType, _LAYOUT_MAPPING_HANDLERS);
}
function _cleanupSite(aType) {
    _cleanupAsset(aType, _SITE_HANDLERS);
}
function _cleanupPage(aType) {
    _cleanupAsset(aType, _PAGE_HANDLERS);
}
function _cleanupContent(aType) {
    if (lodash_1.isString(aType.typeId)) {
        delete aType.type;
    }
    _cleanupAsset(aType, _CONTENT_HANDLERS);
}
var _DELETE = ['rev', 'creatorId', 'lastModifierId', 'created', 'lastModified', 'creator', 'lastModifier', 'links', 'cognitive'];
/**
 * Tests for a plain object
 *
 * @param aObject   the object to test
 */
function _isPlainObject(aObject) {
    return aObject && !aObject.prototype && (typeof aObject === 'object');
}
function _cleanUp(aData) {
    // delete
    if (_isPlainObject(aData)) {
        switch (aData.classification) {
            case 'content-type':
                _cleanupType(aData);
                break;
            case 'layout':
                _cleanupLayout(aData);
                break;
            case 'layout-mapping':
                _cleanupLayoutMappings(aData);
                break;
            case 'site':
                _cleanupSite(aData);
                break;
            case 'page':
                _cleanupPage(aData);
                break;
            case 'content':
                _cleanupContent(aData);
                break;
        }
    }
    // canonicalize
    return json_1.canonicalizeJSON(revision_utils_1.addRevisionId(aData));
}
function _cleanupJSON(aFile) {
    aFile.data = _cleanUp(aFile.data);
    return aFile;
}
var CleanupAssetsCommand = /** @class */ (function (_super) {
    __extends(CleanupAssetsCommand, _super);
    function CleanupAssetsCommand(options) {
        return _super.call(this, options) || this;
    }
    CleanupAssetsCommand.prototype.process = function () {
        // data dir
        var rxDataDir = this.rxMkdirp(this.getDataDir());
        // process
        return rxDataDir.pipe(operators_1.mergeMap(rx_walk_1.walkFiles), operators_1.filter(function (f) { return mime_1.getType(f.absPath) === 'application/json'; }), operators_1.mergeMap(rx_json_1.readJSON), operators_1.map(_cleanupJSON), operators_1.mergeMap(rx_json_1.writeJSON), operators_1.map(function (file) { return file.absPath; }));
    };
    CleanupAssetsCommand.TYPE = 'cleanup';
    CleanupAssetsCommand.COMMAND = 'assets';
    return CleanupAssetsCommand;
}(abstract_command_1.AbstractCommand));
exports.CleanupAssetsCommand = CleanupAssetsCommand;
