"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
/* Copyright IBM Corp. 2017 */
var path_1 = require("path");
var operators_1 = require("rxjs/operators");
var asset_dependencies_1 = require("../utils/asset.dependencies");
var file_utils_1 = require("../utils/file.utils");
var rx_copy_assets_1 = require("../utils/rx.copy.assets");
var rx_file_1 = require("../utils/rx.file");
var rx_utils_1 = require("../utils/rx.utils");
var rx_walk_1 = require("../utils/rx.walk");
var constants_1 = require("./../constants");
var options_1 = require("./../options");
var rx_copy_1 = require("./../utils/rx.copy");
var rx_delete_1 = require("./../utils/rx.delete");
var wchtools_1 = require("./../utils/wchtools");
var abstract_command_1 = require("./abstract.command");
var help_1 = require("./help");
function _copyAggregated(aPackageFile, aDstDir, aLogger) {
    var pkgFile = aPackageFile;
    // copy the dependencies
    var dstBase = aDstDir;
    aLogger.next("Aggregating to [" + dstBase + "].");
    // clean up after execution
    var rxDelete = rx_delete_1.rxDeleteDir(dstBase);
    // copy the dependencies
    var rxCopy = asset_dependencies_1.rxCopyDependencies(pkgFile, dstBase);
    return rxCopy;
}
function _copyDirect(aSrcDir, aDstDir, aLogger) {
    // log this
    aLogger.next("Copying from [" + aSrcDir + "] to [" + aDstDir + "].");
    // copy source files
    return rx_copy_1.rxCopyDir(aSrcDir, aDstDir);
}
function _writeOptions(aTools, aDstDir) {
    var _a;
    // prepare the context
    var data = (_a = {},
        _a[wchtools_1.KEY_API_URL] = aTools.baseUrl,
        _a[wchtools_1.KEY_USERNAME] = aTools.username,
        _a);
    // write the file
    return rx_walk_1.rxMkdirp(aDstDir).pipe(operators_1.mergeMap(function (dir) { return rx_file_1.writeJsonFileSafe(path_1.join(dir, wchtools_1.FILE_WCHTOOLS_JSON), data, true); }));
}
/**
 * Copies the build output to the correct location
*/
var CopyContentCommand = /** @class */ (function (_super) {
    __extends(CopyContentCommand, _super);
    function CopyContentCommand(options) {
        return _super.call(this, options) || this;
    }
    CopyContentCommand.prototype.printHelp = function () {
        // log the command line
        console.log(CopyContentCommand.COMMAND, CopyContentCommand.TYPE, "--" + constants_1.OPTION_PKG_DIR + " <PKG>", "--" + constants_1.OPTION_AGGREGATED, "--" + constants_1.OPTION_DATA + " <DATA>", "--" + constants_1.OPTION_OUT + " <OUT>", "[--" + constants_1.OPTION_USER + " <USER>]", "[--" + constants_1.OPTION_PASSWORD + " <PWD>]", "[--" + constants_1.OPTION_URL + " <URL>]", "[--" + constants_1.OPTION_OVERRIDE + "]", "[--" + constants_1.OPTION_VERBOSE + "]");
        // log the parameters
        console.log("--" + constants_1.OPTION_PKG_DIR + " <PKG>", ':', help_1.HELP_PKG_DIR);
        console.log("--" + constants_1.OPTION_AGGREGATED, ':', help_1.HELP_AGGREGATED);
        console.log("--" + constants_1.OPTION_OUT + " <OUT>", ':', help_1.HELP_DST_DIR);
        console.log("--" + constants_1.OPTION_DATA + " <DATA>", ':', help_1.HELP_DATA_DIRECTORY);
        console.log("--" + constants_1.OPTION_USER + " <USER>", ':', help_1.HELP_USERNAME);
        console.log("--" + constants_1.OPTION_PASSWORD + " <PWD>", ':', help_1.HELP_PASSWORD);
        console.log("--" + constants_1.OPTION_URL + " <URL>", ':', help_1.HELP_API_URL);
        console.log("--" + constants_1.OPTION_OVERRIDE, ':', help_1.HELP_OVERRIDE);
        console.log("--" + constants_1.OPTION_VERBOSE, ':', help_1.HELP_VERBOSE);
    };
    CopyContentCommand.prototype.process = function () {
        // the options
        var opts = this.getOptions();
        // logger
        var logger = this.getLogger();
        // check if we are aggregated
        var bAggregated = !!opts.aggregated;
        // target directory
        var dstDir = this.getDataDir();
        // source
        var srcDir = options_1.optNormalizeDir(file_utils_1.firstOf(opts.src) || '.');
        // wchtools options
        var rxTools = rx_utils_1.rxCacheSingle(wchtools_1.readWchTools(srcDir));
        // matcher
        var matcher = wchtools_1.wchToolsGetMatcher(opts);
        // log this
        logger.next("Copying from [" + srcDir + "] to [" + dstDir + "] ...");
        // concatenate
        var rxSourceFiles = rxTools.pipe(operators_1.mergeMap(function (tools) { return rx_copy_assets_1.rxGetContentDependencies(matcher, tools); }));
        // copy
        return rx_copy_1.rxCopyFiles(rxSourceFiles, dstDir);
    };
    CopyContentCommand.TYPE = 'content';
    CopyContentCommand.COMMAND = constants_1.CMD_COPY;
    return CopyContentCommand;
}(abstract_command_1.AbstractCommand));
exports.CopyContentCommand = CopyContentCommand;
