"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
/* Copyright IBM Corp. 2017 */
var lodash_1 = require("lodash");
var path_1 = require("path");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var util_1 = require("util");
var file_utils_1 = require("../utils/file.utils");
var constants_1 = require("./../constants");
var options_1 = require("./../options");
var assert_1 = require("./../utils/assert");
var transliteration_1 = require("./../utils/transliteration");
var abstract_command_1 = require("./abstract.command");
var create_components_command_1 = require("./create.components.command");
var create_layouts_command_1 = require("./create.layouts.command");
var help_1 = require("./help");
function _createLayoutNameMapper(aLayoutName) {
    return function (aNumber, aTotal, aTypeName) { return transliteration_1.camelCase(aLayoutName); };
}
/**
 * create layout --data <DATA> --src <SRC> --type <TYPE> [--name <NAME>] [--verbose] [--flat] [--scss]
 */
var CreateLayoutCommand = /** @class */ (function (_super) {
    __extends(CreateLayoutCommand, _super);
    function CreateLayoutCommand(options) {
        var _this = _super.call(this, options) || this;
        _this.options = options;
        return _this;
    }
    CreateLayoutCommand.prototype.printHelp = function () {
        // log the command line
        console.log(CreateLayoutCommand.COMMAND, CreateLayoutCommand.TYPE, "--" + constants_1.OPTION_SOURCE + " <SOURCE>", "--" + constants_1.OPTION_DATA + " <DATA>", "--" + constants_1.OPTION_TYPE + " <TYPE>", "[--" + constants_1.OPTION_NAME + " <NAME>]", "[--" + constants_1.OPTION_FLAT + "]", "[--" + constants_1.OPTION_SCSS + "]", "[--" + constants_1.OPTION_EDITABLE + "]", "[--" + constants_1.OPTION_VERBOSE + "]");
        // log the parameters
        console.log("--" + constants_1.OPTION_SOURCE + " <SOURCE>", ':', help_1.HELP_COMPONENT_SOURCE);
        console.log("--" + constants_1.OPTION_DATA + " <DATA>", ':', help_1.HELP_DATA_DIRECTORY);
        console.log("--" + constants_1.OPTION_TYPE + " <TYPE>", ':', help_1.HELP_LAYOUT_TYPE);
        console.log("--" + constants_1.OPTION_NAME + " <NAME>", ':', help_1.HELP_LAYOUT_NAME);
        console.log("--" + constants_1.OPTION_FLAT, ':', help_1.HELP_FLAT);
        console.log("--" + constants_1.OPTION_SCSS, ':', help_1.HELP_SCSS);
        console.log("--" + constants_1.OPTION_EDITABLE, ':', help_1.HELP_EDITABLE);
        console.log("--" + constants_1.OPTION_VERBOSE, ':', help_1.HELP_VERBOSE);
    };
    CreateLayoutCommand.prototype.process = function () {
        var _this = this;
        // defer
        return rxjs_1.defer(function () {
            // current options
            var opts = _this.options;
            // the directories
            var dataDirOpt = _this.getDataDir();
            var srcDirOpt = path_1.resolve(file_utils_1.firstOf(opts.src) || _this.getAppDir());
            // fallback source directories
            var srcDirs = (file_utils_1.arrayOf(opts.src) || [srcDirOpt]).map(options_1.optNormalizeDir);
            // construct the directories
            var rxDataDir = _this.rxMkdirp(dataDirOpt);
            var rxSrcDir = _this.rxMkdirp(srcDirOpt).pipe(operators_1.shareReplay(1));
            var rxSrcDirs = rxjs_1.concat.apply(void 0, srcDirs.map(function (dir) { return _this.rxMkdirp(dir); })).pipe(operators_1.toArray(), operators_1.shareReplay(1));
            // logger
            var logger = _this.getLogger();
            // validate the existence of the type attribute
            assert_1.assertParameter(opts.type, constants_1.OPTION_TYPE);
            assert_1.assertNotNull(dataDirOpt, constants_1.OPTION_DATA);
            assert_1.assertNotNull(srcDirOpt, constants_1.OPTION_SOURCE);
            // type pattern
            var includePattern = "^" + lodash_1.escapeRegExp(opts.type) + "$";
            // name of the layout
            var mapper = util_1.isString(opts.name) ? _createLayoutNameMapper(opts.name) : undefined;
            // log the name
            logger.next("Data dir [" + dataDirOpt + "], source dir [" + srcDirOpt + "], type [" + opts.type + "], name [" + opts.name + "].");
            // log the pattern
            logger.next("Using regular expression [" + includePattern + "].");
            // create the mappings
            var createLayoutsOpts = rxDataDir.pipe(operators_1.map(function (aDataDir) { return ({
                command: create_layouts_command_1.CreateLayoutsCommand.COMMAND,
                commandType: create_layouts_command_1.CreateLayoutsCommand.TYPE,
                data: aDataDir,
                verbose: opts.verbose,
                flat: opts.flat,
                module: opts.module,
                modules: opts.modules,
                include: includePattern,
                count: 1,
                layoutNameMapper: mapper,
                allArgs: opts.allArgs
            }); }));
            // create the components
            var createComponentsOpts = rxjs_1.combineLatest([rxDataDir, rxSrcDirs]).pipe(operators_1.map(function (_a) {
                var aDataDir = _a[0], aSrcDirs = _a[1];
                return ({
                    command: create_components_command_1.CreateComponentsCommand.COMMAND,
                    commandType: create_components_command_1.CreateComponentsCommand.TYPE,
                    data: aDataDir,
                    include: includePattern,
                    src: aSrcDirs,
                    scss: opts.scss,
                    flat: opts.flat,
                    module: opts.module,
                    modules: opts.modules,
                    skipRegistration: opts.skipRegistration,
                    layoutHtmlTemplate: opts.layoutHtmlTemplate,
                    editable: opts.editable,
                    verbose: opts.verbose,
                    allArgs: opts.allArgs
                });
            }));
            // construct the layouts
            var rxLayouts = createLayoutsOpts.pipe(operators_1.map(function (o) { return new create_layouts_command_1.CreateLayoutsCommand(o); }), operators_1.mergeMap(function (cmd) { return cmd.process(); }));
            // construct the components
            var rxComponents = createComponentsOpts.pipe(operators_1.map(function (o) { return new create_components_command_1.CreateComponentsCommand(o); }), operators_1.mergeMap(function (cmd) { return cmd.process(); }));
            // chain
            return rxjs_1.concat(rxLayouts, rxComponents);
        });
    };
    CreateLayoutCommand.TYPE = 'layout';
    CreateLayoutCommand.COMMAND = constants_1.CMD_CREATE;
    return CreateLayoutCommand;
}(abstract_command_1.AbstractCommand));
exports.CreateLayoutCommand = CreateLayoutCommand;
