"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
/* Copyright IBM Corp. 2017 */
var Promise = require("bluebird");
var child_process_1 = require("child_process");
var lodash_1 = require("lodash");
var path_1 = require("path");
var process_1 = require("process");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var stream_1 = require("stream");
var url_1 = require("url");
var file_utils_1 = require("../utils/file.utils");
var rx_file_1 = require("../utils/rx.file");
var rx_hbs_1 = require("../utils/rx.hbs");
var url_utils_1 = require("../utils/url.utils");
var constants_1 = require("./../constants");
var options_1 = require("./../options");
var assert_1 = require("./../utils/assert");
var json_1 = require("./../utils/json");
var rx_delete_1 = require("./../utils/rx.delete");
var rx_walk_1 = require("./../utils/rx.walk");
var transliteration_1 = require("./../utils/transliteration");
var wchtools_1 = require("./../utils/wchtools");
var abstract_command_1 = require("./abstract.command");
var add_base_command_1 = require("./add.base.command");
var add_esi_polyfill_command_1 = require("./add.esi.polyfill.command");
var add_preload_command_1 = require("./add.preload.command");
var add_webappmanifest_command_1 = require("./add.webappmanifest.command");
var application_pull_command_1 = require("./application.pull.command");
var application_push_command_1 = require("./application.push.command");
var build_development_command_1 = require("./build.development.command");
var build_production_command_1 = require("./build.production.command");
var compress_assets_command_1 = require("./compress.assets.command");
var copy_build_command_1 = require("./copy.build.command");
var create_components_command_1 = require("./create.components.command");
var create_layouts_command_1 = require("./create.layouts.command");
var create_page_type_command_1 = require("./create.page.type.command");
var create_site_command_1 = require("./create.site.command");
var create_webappmanifest_command_1 = require("./create.webappmanifest.command");
var help_1 = require("./help");
var normalize_json_command_1 = require("./normalize.json.command");
var run_wchtools_command_1 = require("./run.wchtools.command");
var show_types_command_1 = require("./show.types.command");
var show_url_command_1 = require("./show.url.command");
var cli = require('@angular/cli')["default"];
var notice = /create\s+([^\s]+)/g;
var noColors = /\x1B\[([0-9]{1,2}(;[0-9]{1,2})?)?[mGK]/g;
var ASSETS_BASE = path_1.join(__dirname, '..', '..', '..', 'assets');
function _isInternalHost(aBaseURL) {
    var url = url_1.parse(aBaseURL);
    return !!(url.host && !url.host.startsWith('www.') && (url.host.indexOf('digitalexperience.ibm.com') <= 0));
}
function _pullData(aDataDir, aConfig, aOptions, aLogger) {
    // build the command line
    var args = [
        path_1.normalize(path_1.join(__dirname, '../utils/fork.wchtools')),
        'pull',
        '--dir',
        aDataDir,
        '--user',
        aConfig.username,
        '--password',
        aConfig.password,
        '--url',
        aConfig.baseUrl,
        '-tlmsp'
    ];
    // verbose flag
    if (aOptions.verbose) {
        args.push('--verbose');
    }
    // execute
    return rxjs_1.Observable.create(function (o) {
        // spawn the process
        var childProcess = child_process_1.spawn(process.execPath, args, {
            'cwd': aDataDir,
            'env': Object.assign({
                'NODE_TLS_REJECT_UNAUTHORIZED': '0'
            }, process_1.env)
        });
        // start
        o.next('start');
        // attach
        childProcess.stdout.on('data', function (data) { return aLogger.next(data.toString().replace(noColors, '')); });
        childProcess.stderr.on('data', function (data) { return aLogger.next(data.toString().replace(noColors, '')); });
        // end
        childProcess.on('close', function (code) { return o.complete(); });
        childProcess.on('error', function (err) { return o.error(err); });
    });
}
var OutputParser = /** @class */ (function (_super) {
    __extends(OutputParser, _super);
    function OutputParser(aBaseDir) {
        var _this = _super.call(this) || this;
        _this.aBaseDir = aBaseDir;
        _this.files = new rxjs_1.Subject();
        return _this;
    }
    OutputParser.prototype._write = function (chunk, encoding, callback) {
        // extract
        var strg = chunk.toString().replace(noColors, '').trim();
        var res = notice.exec(strg);
        if (res && (res.length > 1)) {
            // dump
            this.files.next(path_1.join(this.aBaseDir, res[1]));
        }
        callback();
    };
    OutputParser.prototype._final = function (callback) {
        console.log('done');
        _super.prototype._final.call(this, callback);
    };
    return OutputParser;
}(stream_1.Writable));
function _pullTypes(aSrcDir, aName, aConfig, aOptions, aLogger) {
    // just pull once
    var dataDir = path_1.join(aSrcDir, aName, 'data');
    // log this
    aLogger.next("Pulling types to [" + dataDir + "].");
    // continue
    return rx_walk_1.rxMkdirp(dataDir).pipe(operators_1.mergeMap(function (dir) { return _pullData(dir, aConfig, aOptions, aLogger).pipe(operators_1.last(), operators_1.mergeMap(function () { return rx_walk_1.walkFiles(dir).pipe(operators_1.map(function (desc) { return desc.absPath; })); })); }), operators_1.shareReplay(1));
}
function _createLayoutsAndMappings(aSrcDir, aName, aOptions, aLogger) {
    // the data dir
    var dataDir = path_1.join(aSrcDir, aName, 'data');
    // execute the layouts command
    var opt = {
        command: create_layouts_command_1.CreateLayoutsCommand.COMMAND,
        commandType: create_layouts_command_1.CreateLayoutsCommand.TYPE,
        data: dataDir,
        verbose: aOptions.verbose,
        override: aOptions.override,
        include: aOptions.include,
        exclude: aOptions.exclude,
        allArgs: aOptions.allArgs
    };
    // create
    return rxjs_1.of(opt).pipe(operators_1.map(function (o) { return new create_layouts_command_1.CreateLayoutsCommand(opt); }), operators_1.mergeMap(function (cmd) { return cmd.process(); }), operators_1.shareReplay());
}
function _createComponents(aSrcDir, aName, aOptions, aLogger) {
    // the data dir
    var dataDir = path_1.join(aSrcDir, aName, 'data');
    var srcDir = path_1.join(aSrcDir, aName, 'src', 'app');
    // execute the layouts command
    var opt = {
        command: create_components_command_1.CreateComponentsCommand.COMMAND,
        commandType: create_components_command_1.CreateComponentsCommand.TYPE,
        data: dataDir,
        src: srcDir,
        verbose: aOptions.verbose,
        override: aOptions.override,
        include: aOptions.include,
        exclude: aOptions.exclude,
        editable: aOptions.editable,
        skipRegistration: true,
        scss: true,
        allArgs: aOptions.allArgs
    };
    // process
    return rxjs_1.of(opt).pipe(operators_1.map(function (o) { return new create_components_command_1.CreateComponentsCommand(opt); }), operators_1.mergeMap(function (cmd) { return cmd.process(); }), operators_1.shareReplay());
}
function _createNgCli(aSrcDir, aName, aLogger) {
    // sanity check
    assert_1.assertNotNull(aSrcDir, 'srcDir');
    assert_1.assertNotNull(aName, 'name');
    // trigger
    return rxjs_1.defer(function () { return rxjs_1.of(aSrcDir).pipe(operators_1.mergeMap(function (srcDir) {
        // notify the start
        aLogger.next('createNgCli start');
        // parse the command output
        var outParser = new OutputParser(path_1.normalize(srcDir));
        // the files handler
        var files = outParser.files;
        // log the arguments
        var cliArgs = ['new', aName, '--skip-install', '--skip-git', '--skip-commit', '--routing', '--style', 'scss'];
        // notify the start
        aLogger.next("Calling CLI with " + JSON.stringify(cliArgs));
        // execute the command, we will sync on files
        Promise.resolve(cli({
            cliArgs: cliArgs,
            directory: srcDir,
            inputStream: process.stdin,
            outputStream: outParser
        }))
            .then(function () { return files.complete(); })["catch"](function (err) { return files.error(err); });
        // returns the generated files
        return files.asObservable();
    }), operators_1.finalize(function () { return aLogger.next('createNgCli end'); }), operators_1.shareReplay()); });
}
exports.createNgCli = _createNgCli;
function _log(aData) {
    console.log('CreateProjectCommand', aData);
}
function _createAppConfig(aSourceDir, aDataDir, aOverride) {
    // compute the relative path
    var dstDir = path_1.join(aSourceDir, 'src', 'app');
    var dstFile = path_1.join(dstDir, 'app.config.ts');
    var relPath = lodash_1.replace(path_1.relative(dstDir, aDataDir), /\\/g, '/');
    // prepare the hbs context
    var ctx = {
        dataDir: "./" + relPath + "/"
    };
    // load the template
    var rxTemplate = rx_hbs_1.hbsCompile(path_1.join(ASSETS_BASE, 'app.config.ts.hbs'));
    // transform
    return rxTemplate.pipe(operators_1.map(function (tmp) { return tmp(ctx); }), operators_1.mergeMap(function (content) { return rx_file_1.writeTextFileSafe(dstFile, content, aOverride); }));
}
exports.createProjectCreateAppConfig = _createAppConfig;
function _fixAppModule(aSource) {
    // insert the layouts import
    var src = "import { LAYOUTS } from './layouts';\nimport { WchNgModule } from '@ibm-wch-sdk/ng';\nimport { environment } from '../environments/environment';\n" + aSource;
    // add the module to the end of the declarations
    src = src.replace(/(declarations:\s+\[)([^\]]+)(\])/g, '$1$2, ...LAYOUTS$3');
    // add the entry components
    src = src.replace(/(bootstrap:)/g, 'entryComponents: [...LAYOUTS],\n$1');
    // add the SDK
    src = src.replace(/(imports:\s+\[)([^\]]+)(\])/g, '$1$2, WchNgModule.forRoot(environment)$3');
    return src;
}
function _rewriteAppModule(aSrcDir, aName, aLogger) {
    // name
    var name = path_1.join(aSrcDir, aName, 'src', 'app', constants_1.APP_MODULE_FILE);
    return rx_walk_1.readFile(name).pipe(operators_1.map(_fixAppModule), operators_1.mergeMap(function (data) { return rx_walk_1.writeFile(name, data); }));
}
var envRegExp = /(environment\s+=\s+\{)([^\}]+)(\})/g;
function _fixDevEnvironment(aSource, aApiUrl) {
    // insert the layouts import
    var src = aSource;
    // prefix
    var prefix = 'import { apiUrl } from \'./../app/app.config\';\n';
    // fragment
    var fragment = ", apiUrl\n, httpOptions: { pollTime: 20 * 1000 }, httpPreviewOptions: { pollTime: 10 * 1000 }";
    // add the delivery and api URL
    src = prefix + src.replace(envRegExp, '$1$2' + fragment + '$3');
    return src;
}
function _fixProdEnvironment(aSource, aApiUrl) {
    // insert the layouts import
    var src = aSource;
    // deduce the delivery URL from the api URL
    var deliveryUrl = wchtools_1.wchDeliveryUrlFromApiUrl(aApiUrl);
    // fragment
    var fragment = ", httpOptions: { pollTime: 5 * 60 * 1000 }, httpPreviewOptions: { pollTime: 10 * 1000 }";
    // add the delivery and api URL
    src = src.replace(envRegExp, '$1$2' + fragment + '$3');
    return src;
}
function _rewriteEnvironment(aSrcDir, aName, aApiUrl, aLogger) {
    // dev env
    var devName = path_1.join(aSrcDir, aName, 'src', 'environments', 'environment.ts');
    var dev = rx_walk_1.readFile(devName).pipe(operators_1.map(function (data) { return _fixDevEnvironment(data, aApiUrl); }), operators_1.mergeMap(function (data) { return rx_walk_1.writeFile(devName, data); }));
    // dev env
    var prodEnv = path_1.join(aSrcDir, aName, 'src', 'environments', 'environment.prod.ts');
    var prod = rx_walk_1.readFile(prodEnv).pipe(operators_1.map(function (data) { return _fixProdEnvironment(data, aApiUrl); }), operators_1.mergeMap(function (data) { return rx_walk_1.writeFile(prodEnv, data); }));
    // merge
    return rxjs_1.merge(dev, prod);
}
function _fixRoutingModule(aSource, aTools) {
    // find the first page
    var defaultRoute = '/home/';
    if (aTools && aTools.sites && aTools.sites["default"]) {
        // access the default site
        var defSite = aTools.sites["default"];
        if (defSite.children && (defSite.children.length > 0)) {
            // use the first child as default, do not encode this (strange!)
            defaultRoute = "/" + defSite.children[0].data.segment + "/";
        }
    }
    // insert the layouts import
    var src = "import { PageComponent } from '@ibm-wch-sdk/ng';\n" + aSource;
    // add the module to the end of the declarations
    var replace = "{\n        path: '',\n        redirectTo: '" + defaultRoute + "',\n        pathMatch: 'full'\n      },\n      {\n        path: '**',\n        component: PageComponent\n      }";
    src = src.replace(/(const\s+routes:\s+Routes\s+=\s+\[)([^;]*)(\];)/g, '$1' + replace + '$3');
    // use hash based navigation
    var hashRegexp = /(RouterModule\.forRoot\s*\()([^\)]*)(\))/g;
    src = src.replace(hashRegexp, '$1$2, { useHash: true }$3');
    return src;
}
function _rewriteRoutingModule(aSrcDir, aName, aLogger) {
    // data directory
    var dataDir = path_1.join(aSrcDir, aName, 'data');
    // name
    var name = path_1.join(aSrcDir, aName, 'src', 'app', 'app-routing.module.ts');
    return rx_walk_1.readFile(name).pipe(operators_1.mergeMap(function (code) { return wchtools_1.readWchTools(dataDir).pipe(operators_1.map(function (tools) { return _fixRoutingModule(code, tools); })); }), operators_1.mergeMap(function (data) { return rx_walk_1.writeFile(name, data); }));
}
function _fixIndex(aSource) {
    // insert the layouts import
    var src = aSource;
    // add the module to the end of the declarations
    src = src.replace(/(<base\s+href=\")([^\"]*)(\">)/g, '$1.$3');
    return src;
}
function _rewriteIndex(aSrcDir, aName, aLogger) {
    // name
    var name = path_1.join(aSrcDir, aName, 'src', 'index.html');
    return rx_walk_1.readFile(name).pipe(operators_1.map(_fixIndex), operators_1.mergeMap(function (data) { return rx_walk_1.writeFile(name, data); }));
}
function _fixLint(aLint) {
    // need this due to still unresolved https://github.com/mgechev/codelyzer/issues/191
    var rules = aLint.rules || (aLint.rules = {});
    rules['no-access-missing-member'] = false;
    // SDK uses 'on' as prefix
    rules['no-output-on-prefix'] = false;
    // add rxjs rules
    var ext = aLint["extends"] || (aLint["extends"] = []);
    ext.push('rxjs-tslint-rules');
    // add extra rules
    rules['rxjs-add'] = { 'severity': 'error' };
    rules['rxjs-no-unused-add'] = { 'severity': 'error' };
    rules['rxjs-no-subject-unsubscribe'] = { 'severity': 'error' };
    // ok
    return aLint;
}
function _rewriteLint(aSrcDir, aName, aLogger) {
    // name
    var name = path_1.join(aSrcDir, aName, 'tslint.json');
    return rx_walk_1.readFile(name).pipe(operators_1.map(function (data) { return JSON.parse(data); }), operators_1.map(_fixLint), operators_1.map(json_1.canonicalizeJSON), operators_1.map(function (data) { return JSON.stringify(data, undefined, 2); }), operators_1.mergeMap(function (data) { return rx_walk_1.writeFile(name, data); }));
}
function _fixCli(aCli) {
    // add the flag to all fields
    var apps = aCli.apps || (aCli.apps = []);
    // iterate and add
    apps.forEach(function (app) { return app.serviceWorker = false; });
    // ok
    return aCli;
}
function _rewriteCli(aSrcDir, aName, aLogger) {
    // name
    var name = path_1.join(aSrcDir, aName, '.angular-cli.json');
    return rx_walk_1.readFile(name).pipe(operators_1.map(function (data) { return JSON.parse(data); }), operators_1.map(_fixCli), operators_1.map(json_1.canonicalizeJSON), operators_1.map(function (data) { return JSON.stringify(data, undefined, 2); }), operators_1.mergeMap(function (data) { return rx_walk_1.writeFile(name, data); }));
}
function _createWchtoolsOptions(aDataDir, aConfig, aLogger) {
    // build the object
    var opts = {
        'username': aConfig.username,
        'x-ibm-dx-tenant-base-url': aConfig.baseUrl,
        'ignore_additive': true
    };
    // serialize and write
    var name = path_1.join(aDataDir, wchtools_1.FILE_WCHTOOLS_JSON);
    // construct the data directory
    return rx_walk_1.rxMkdirp(aDataDir).pipe(operators_1.mergeMapTo(rx_walk_1.writeFile(name, JSON.stringify(json_1.canonicalizeJSON(opts), undefined, 2))));
}
exports.createProjectCreateWchtoolsOptions = _createWchtoolsOptions;
function _getDataDir(aSrcDir, aName) {
    return path_1.join(aSrcDir, aName, 'data');
}
function _updatePackage(aPackage, aName, bEditable) {
    // add the dependencies
    var dependencies = aPackage.dependencies;
    var devDependencies = aPackage.devDependencies;
    // add service workers
    if (!devDependencies['@angular/service-worker']) {
        devDependencies['@angular/service-worker'] = '^6.0.0';
    }
    // SDK dependencies
    dependencies['@ibm-wch-sdk/ng'] = '^6.0.0';
    devDependencies['@ibm-wch-sdk/cli'] = '^6.0.0';
    // add dependency if editable
    if (bEditable) {
        dependencies['@ibm-wch-sdk/ng-edit'] = '^6.0.0';
    }
    // some RX stuff
    devDependencies['rxjs-tslint-rules'] = '^4.1.1';
    devDependencies['cpx'] = '^1.5.0';
    devDependencies['rimraf'] = '^2.6.2';
    devDependencies['npm-run-all'] = '^4.1.3';
    // engines
    aPackage.engines = {
        'node': '>= 7.0',
        'npm': '>= 5.0'
    };
    // config
    var cfg = aPackage.config = {};
    cfg[constants_1.OPTION_DATA] = './data';
    cfg[constants_1.OPTION_SOURCE] = './src';
    cfg[constants_1.OPTION_NAME] = aName;
    cfg[constants_1.OPTION_ROOT] = '/';
    var CLI = 'ibm-wch-sdk-cli';
    // add scripts
    var scripts = aPackage.scripts;
    scripts['build:production'] = CLI + " " + build_production_command_1.BuildProductionCommand.COMMAND + " " + build_production_command_1.BuildProductionCommand.TYPE + " --" + constants_1.OPTION_RELATIVE;
    // add scripts
    scripts['build:development'] = CLI + " " + build_development_command_1.BuildDevelopmentCommand.COMMAND + " " + build_development_command_1.BuildDevelopmentCommand.TYPE + " --" + constants_1.OPTION_RELATIVE;
    scripts['run:wchtools'] = CLI + " " + run_wchtools_command_1.RunWchToolsCommand.COMMAND + " " + run_wchtools_command_1.RunWchToolsCommand.TYPE;
    scripts['gen:layouts'] = CLI + " " + create_layouts_command_1.CreateLayoutsCommand.COMMAND + " " + create_layouts_command_1.CreateLayoutsCommand.TYPE + " --" + constants_1.OPTION_DATA + " " + cfg[constants_1.OPTION_DATA] + "/";
    scripts['gen:components'] = CLI + " " + create_components_command_1.CreateComponentsCommand.COMMAND + " " + create_components_command_1.CreateComponentsCommand.TYPE + " --" + constants_1.OPTION_SOURCE + " " + cfg[constants_1.OPTION_SOURCE] + "/app/ --" + constants_1.OPTION_DATA + " " + cfg[constants_1.OPTION_DATA] + " --" + constants_1.OPTION_SCSS;
    scripts['package:copyResources'] = CLI + " " + copy_build_command_1.CopyBuildCommand.COMMAND + " " + copy_build_command_1.CopyBuildCommand.TYPE;
    scripts['package:addPrefetch'] = CLI + " " + add_preload_command_1.AddPreloadCommand.COMMAND + " " + add_preload_command_1.AddPreloadCommand.TYPE + " --" + constants_1.OPTION_RELATIVE;
    scripts['package:addEsiPolyfill'] = CLI + " " + add_esi_polyfill_command_1.AddEsiPolyfillCommand.COMMAND + " " + add_esi_polyfill_command_1.AddEsiPolyfillCommand.TYPE;
    scripts['package:createSite'] = CLI + " " + create_site_command_1.CreateSiteCommand.COMMAND + " " + create_site_command_1.CreateSiteCommand.TYPE;
    scripts['package:compress'] = CLI + "     " + compress_assets_command_1.CompressAssetsCommand.COMMAND + "     " + compress_assets_command_1.CompressAssetsCommand.TYPE;
    scripts['package:production'] = "npm run package:copyResources && npm run package:addPrefetch && npm run package:addBaseUrl && npm run package:addManifest && npm run package:addEsiPolyfill && npm run package:createSite";
    scripts['deploy:push'] = CLI + " " + application_push_command_1.ApplicationPushCommand.COMMAND + " " + application_push_command_1.ApplicationPushCommand.TYPE + " --" + constants_1.OPTION_DATA + " " + cfg[constants_1.OPTION_DATA] + "/";
    scripts['gen:pull'] = CLI + " " + application_pull_command_1.ApplicationPullCommand.COMMAND + " " + application_pull_command_1.ApplicationPullCommand.TYPE + " --" + constants_1.OPTION_DATA + " " + cfg[constants_1.OPTION_DATA] + "/";
    scripts['gen:pagetype'] = CLI + " " + create_page_type_command_1.CreatePageTypeCommand.COMMAND + " " + create_page_type_command_1.CreatePageTypeCommand.TYPE + " --" + constants_1.OPTION_SOURCE + " " + cfg[constants_1.OPTION_SOURCE] + "/app/ --" + constants_1.OPTION_DATA + " " + cfg[constants_1.OPTION_DATA] + " --" + constants_1.OPTION_SCSS + " --" + constants_1.OPTION_TYPE;
    scripts['show:types'] = CLI + " " + show_types_command_1.ShowTypesCommand.COMMAND + " " + show_types_command_1.ShowTypesCommand.TYPE;
    scripts['show:url'] = CLI + " " + show_url_command_1.ShowUrlCommand.COMMAND + " " + show_url_command_1.ShowUrlCommand.TYPE;
    scripts['package:addBaseUrl'] = CLI + " " + add_base_command_1.AddBaseCommand.COMMAND + " " + add_base_command_1.AddBaseCommand.TYPE + " --" + constants_1.OPTION_RELATIVE;
    scripts['package:addManifest'] = CLI + " " + add_webappmanifest_command_1.AddWebAppManifestCommand.COMMAND + " " + add_webappmanifest_command_1.AddWebAppManifestCommand.TYPE + " --" + constants_1.OPTION_RELATIVE;
    scripts['gen:normalize'] = CLI + " " + normalize_json_command_1.NormalizeJsonCommand.COMMAND + " " + normalize_json_command_1.NormalizeJsonCommand.TYPE;
    scripts['deploy'] = 'npm run build:production && npm run package:production && npm run deploy:push && npm run show:url';
    scripts['lint'] = 'ng lint --type-check';
    return aPackage;
}
function _updatePackageJson(aSrcDir, aName, aEditable, aLogger) {
    // load the package json
    var name = path_1.join(aSrcDir, aName, 'package.json');
    return rx_walk_1.readFile(name).pipe(operators_1.map(function (data) { return JSON.parse(data); }), operators_1.map(function (pkg) { return _updatePackage(pkg, aName, aEditable); }), operators_1.map(json_1.canonicalizeJSON), operators_1.map(function (json) { return JSON.stringify(json, null, 2); }), operators_1.mergeMap(function (pkg) { return rx_walk_1.writeFile(name, pkg); }));
}
function _updateVisualStudioCodeSettings(aSrcDir, aName, aWchtoolsOptions, aLogger) {
    // templates
    var templateFile = path_1.join(ASSETS_BASE, 'vscode.settings.json.template');
    var rxTemplateContent = rx_walk_1.readFile(templateFile).pipe(operators_1.map(function (data) { return JSON.parse(data); }));
    // target path
    var dstDir = path_1.join(aSrcDir, aName, '.vscode');
    var dstFile = path_1.join(dstDir, 'settings.json');
    // construct target file
    var rxDir = rx_walk_1.rxMkdirp(dstDir);
    // rewrite
    var apiUrl = url_utils_1.ensureTrailingSlash(aWchtoolsOptions.baseUrl);
    var userName = aWchtoolsOptions.username;
    // update
    return rxjs_1.combineLatest(rxTemplateContent.pipe(operators_1.map(function (data) { return Object.assign(data, { 'wch.username': userName, 'wch.apiUrl': apiUrl }); })), rxDir).pipe(operators_1.mergeMap(function (_a) {
        var data = _a[0], dir = _a[1];
        return rx_file_1.writeJsonFileSafe(dstFile, data, true);
    }));
}
function _updateVisualStudioCodeLaunch(aSrcDir, aName, aWchtoolsOptions, aLogger) {
    // templates
    var templateFile = path_1.join(ASSETS_BASE, 'vscode.launch.json.template');
    var rxTemplateContent = rx_walk_1.readFile(templateFile).pipe(operators_1.map(function (data) { return JSON.parse(data); }));
    // target path
    var dstDir = path_1.join(aSrcDir, aName, '.vscode');
    var dstFile = path_1.join(dstDir, 'launch.json');
    // construct target file
    var rxDir = rx_walk_1.rxMkdirp(dstDir);
    // get the URLs
    var rxIndex = wchtools_1.wchGetIndexUrl('/', aWchtoolsOptions);
    // update the data
    var rxData = rxjs_1.combineLatest(rxTemplateContent, rxIndex).pipe(operators_1.map(function (_a) {
        var data = _a[0], index = _a[1];
        // update the data
        var cfg = data.configurations || (data.configurations = []);
        // insert a special server side launch config
        cfg.push({
            'name': 'Launch on WCH',
            'type': 'chrome',
            'request': 'launch',
            'url': index
        });
        // ok
        return data;
    }));
    // update
    return rxjs_1.combineLatest(rxData, rxDir, rxIndex).pipe(operators_1.mergeMap(function (_a) {
        var data = _a[0], dir = _a[1], index = _a[2];
        return rx_file_1.writeJsonFileSafe(dstFile, data, true);
    }));
}
function _updateVisualStudioCodeTasks(aSrcDir, aName, aWchtoolsOptions, aLogger) {
    // templates
    var templateFile = path_1.join(ASSETS_BASE, 'vscode.tasks.json.template');
    var rxTemplateContent = rx_walk_1.readFile(templateFile).pipe(operators_1.map(function (data) { return JSON.parse(data); }));
    // target path
    var dstDir = path_1.join(aSrcDir, aName, '.vscode');
    var dstFile = path_1.join(dstDir, 'tasks.json');
    // construct target file
    var rxDir = rx_walk_1.rxMkdirp(dstDir);
    // update the data
    var rxData = rxTemplateContent;
    // update
    return rxjs_1.combineLatest(rxData, rxDir).pipe(operators_1.mergeMap(function (_a) {
        var data = _a[0], dir = _a[1];
        return rx_file_1.writeJsonFileSafe(dstFile, data, true);
    }));
}
function _updateProjectTypings(aTypingsFile, aLogger) {
    // templates
    var templateFile = path_1.join(ASSETS_BASE, 'typings.d.ts.template');
    var rxTemplateContent = rx_walk_1.readFile(templateFile);
    var typingsFile = aTypingsFile;
    var rxTypingsContent = rx_walk_1.readFile(typingsFile);
    return rxjs_1.combineLatest(rxTemplateContent, rxTypingsContent).pipe(operators_1.map(function (_a) {
        var templateContent = _a[0], typingsContent = _a[1];
        return typingsContent + "\n" + templateContent;
    }), operators_1.mergeMap(function (data) { return rx_walk_1.writeFile(typingsFile, data); }));
}
exports.createProjectUpdateTypings = _updateProjectTypings;
function _updateTypings(aSrcDir, aName, aLogger) {
    var typingsFile = path_1.join(aSrcDir, aName, 'src', 'typings.d.ts');
    return _updateProjectTypings(typingsFile, aLogger);
}
function _copyResources(aSrcDir, aName, aOpts, aLogger) {
    // check if we need to copy the npmrc
    var res = [];
    if (_isInternalHost(aOpts.baseUrl)) {
        // copy
        res.push(rx_walk_1.rxCopyFile(path_1.join(ASSETS_BASE, 'npmrc.template'), path_1.join(aSrcDir, aName, '.npmrc')));
        res.push(rx_walk_1.rxCopyFile(path_1.join(ASSETS_BASE, 'yarnrc.template'), path_1.join(aSrcDir, aName, '.yarnrc')));
    }
    res.push(rx_walk_1.rxCopyFile(path_1.join(ASSETS_BASE, 'gitignore.template'), path_1.join(aSrcDir, aName, '.gitignore')));
    res.push(rx_walk_1.rxCopyFile(path_1.join(ASSETS_BASE, 'npmignore.template'), path_1.join(aSrcDir, aName, '.npmignore')));
    // wait for all
    return rxjs_1.merge.apply(void 0, res);
}
exports.createProjectCopyResources = _copyResources;
function _matches(aName, aMatcher, aLogger) {
    // check
    var result = aMatcher(aName);
    aLogger.next("Matching [" + aName + "] results in [" + result + "].");
    return result;
}
/**
 * Selects components by applying a matching function
 *
 * @param aMatcher  the matcher
 * @param aLogger   the logger
 */
function _byMatcher(aMatcher, aLogger) {
    // execute the check
    return function (aType) { return aType && aType.name && _matches(aType.name, aMatcher, aLogger); };
}
function _getFilesToDelete(aTools, aMatcher) {
    // identify all valid types
    var validTypes = {};
    Object.keys(aTools.types).map(function (id) { return aTools.types[id]; })
        .filter(function (type) { return aMatcher(type); })
        .forEach(function (type) { return validTypes[type.id] = type; });
    // check for the valid layout mappings
    var validMappings = {};
    Object.keys(aTools.layoutMappings)
        .map(function (id) { return aTools.layoutMappings[id]; })
        .filter(function (mapping) { return validTypes[mapping.type.id]; })
        .forEach(function (mapping) { return validMappings[mapping.id] = mapping; });
    // check for the valid layouts
    var validLayouts = {};
    Object.keys(validMappings)
        .map(function (id) { return validMappings[id]; })
        .forEach(function (mapping) { return mapping.mappings.forEach(function (m) { return m.layouts.forEach(function (l) { return validLayouts[l.id] = l; }); }); });
    // the invalid types
    var typeFiles = Object.keys(aTools.types)
        .filter(function (id) { return !validTypes[id]; })
        .map(function (id) { return aTools.idToFile[id]; });
    // the invalid mappings
    var mappingFiles = Object.keys(aTools.layoutMappings)
        .filter(function (id) { return !validMappings[id]; })
        .map(function (id) { return aTools.idToFile[id]; });
    // the invalid layouts
    var layoutFiles = Object.keys(aTools.layouts)
        .filter(function (id) { return !validLayouts[id]; })
        .map(function (id) { return aTools.idToFile[id]; });
    // returns the files to delete
    return rxjs_1.merge(rxjs_1.from(typeFiles), rxjs_1.from(mappingFiles), rxjs_1.from(layoutFiles));
}
function _filterWchToolsResources(aSrcDir, aName, aOptions, aLogger) {
    // construct the matcher
    var matcher = wchtools_1.wchToolsGetMatcher(aOptions);
    var typeMatcher = _byMatcher(matcher, aLogger);
    // scan the types
    var dir = path_1.join(aSrcDir, aName, 'data');
    return wchtools_1.readWchTools(dir).pipe(operators_1.mergeMap(function (tools) { return _getFilesToDelete(tools, typeMatcher); }), operators_1.mergeMap(rx_walk_1.rxDeleteFile));
}
function _addManifest(aSrcDir, aName, aOptions) {
    // source
    var srcDir = path_1.join(aSrcDir, aName);
    var dataDir = path_1.join(aSrcDir, aName, 'data');
    // build the options
    return rxjs_1.of({
        command: create_webappmanifest_command_1.CreateWebAppManifestCommand.COMMAND,
        commandType: create_webappmanifest_command_1.CreateWebAppManifestCommand.TYPE,
        data: dataDir,
        src: srcDir,
        name: aName,
        root: aName,
        verbose: aOptions.verbose,
        allArgs: aOptions.allArgs
    }).pipe(operators_1.map(function (opt) { return new create_webappmanifest_command_1.CreateWebAppManifestCommand(opt); }), operators_1.mergeMap(function (cmd) { return cmd.process(); }));
}
/**
 * Generates angular components based on type and layout mapping definitions
 *
 * create project --src SOURCE_CODE_FOLDER --name NAME
 */
var CreateProjectCommand = /** @class */ (function (_super) {
    __extends(CreateProjectCommand, _super);
    function CreateProjectCommand(options) {
        var _this = _super.call(this, options) || this;
        _this.options = options;
        return _this;
    }
    CreateProjectCommand.prototype.printHelp = function () {
        // log the command line
        console.log(CreateProjectCommand.COMMAND, CreateProjectCommand.TYPE, "--" + constants_1.OPTION_SOURCE + " <SOURCE>", "--" + constants_1.OPTION_NAME + " <NAME>", "[--" + constants_1.OPTION_URL + " <URL>]", "[--" + constants_1.OPTION_USER + " <USER>]", "[--" + constants_1.OPTION_PASSWORD + " <PASSWORD>]", "[--" + constants_1.OPTION_INCLUDE_PATTERN + " <INCLUDE_PATTERN>]", "[--" + constants_1.OPTION_EXCLUDE_PATTERN + " <EXCLUDE_PATTERN>]", "[--" + constants_1.OPTION_SKIP_PULL + "]", "[--" + constants_1.OPTION_OVERRIDE + "]", "[--" + constants_1.OPTION_VERBOSE + "]");
        // log the parameters
        console.log("--" + constants_1.OPTION_SOURCE + " <SOURCE>", ':', help_1.HELP_PROJECT_SOURCE);
        console.log("--" + constants_1.OPTION_SKIP_PULL, ':', help_1.HELP_SKIP_PULL);
        console.log("--" + constants_1.OPTION_OVERRIDE, ':', help_1.HELP_PROJECT_OVERRIDE);
        console.log("--" + constants_1.OPTION_VERBOSE, ':', help_1.HELP_VERBOSE);
    };
    CreateProjectCommand.prototype.process = function () {
        var _this = this;
        // options
        var opts = this.options;
        // switch to the target directory
        var rxSrcDir = this.rxMkdirp(options_1.optNormalizeDir(file_utils_1.firstOf(this.options.src) || '.'));
        // capture the current working directory
        var currentDir = process_1.cwd();
        var name = transliteration_1.kebabCase(opts.name || this.getAppName());
        // pull content?
        var bSkipPull = opts.skipPull || false;
        // check if we would override
        var rxClean = (this.options.override && name)
            ? rxSrcDir.pipe(operators_1.mergeMap(function (aSrcDir) { return rx_delete_1.rxDeleteDir(path_1.join(aSrcDir, name)).pipe(operators_1.mergeMap(function () { return rxSrcDir; })); }))
            : rxSrcDir.pipe(operators_1.mergeMap(function (aSrcDir) { return rx_walk_1.rxValidateNotExists(path_1.join(aSrcDir, name)).pipe(operators_1.mergeMap(function () { return rxSrcDir; })); }));
        // logger
        var logger = new rxjs_1.Subject();
        // add some logging
        if (this.options.verbose) {
            logger.subscribe(_log);
        }
        // base URL
        var toolsOpts = this.getWchToolsOptions().pipe(operators_1.shareReplay(1));
        // check if we create an inline edit experience
        var bEditable = opts.editable || false;
        // construct the target
        return rxClean.pipe(operators_1.tap(process_1.chdir), operators_1.mergeMap(function (aSrcDir) {
            // url
            var apiUrl = _this.options.url || '';
            // merge the different steps
            var app = _createNgCli(aSrcDir, name, logger);
            var data = bSkipPull ? app : rxjs_1.combineLatest(toolsOpts, app.pipe(operators_1.take(1))).pipe(operators_1.mergeMap(function (_a) {
                var aToolOpts = _a[0];
                return rxjs_1.concat(bSkipPull ? rxjs_1.empty() : _pullTypes(aSrcDir, name, aToolOpts, _this.options, logger), _filterWchToolsResources(aSrcDir, name, _this.options, logger));
            }));
            var copyResources = toolsOpts.pipe(operators_1.mergeMap(function (aToolsOpts) {
                return _copyResources(aSrcDir, name, aToolsOpts, logger);
            }));
            var wchToolsOptions = toolsOpts.pipe(operators_1.mergeMap(function (aToolsOpts) {
                return _createWchtoolsOptions(_getDataDir(aSrcDir, name), aToolsOpts, logger);
            }));
            var appConfig = _createAppConfig(path_1.join(aSrcDir, name), _getDataDir(aSrcDir, name), !!opts.override);
            var layouts = _createLayoutsAndMappings(aSrcDir, name, _this.options, logger);
            var components = _createComponents(aSrcDir, name, _this.options, logger);
            var pkg = _updatePackageJson(aSrcDir, name, bEditable, logger);
            var typings = _updateTypings(aSrcDir, name, logger);
            var vcSettings = toolsOpts.pipe(operators_1.mergeMap(function (aToolsOpts) { return _updateVisualStudioCodeSettings(aSrcDir, name, aToolsOpts, logger); }));
            var vcLaunch = toolsOpts.pipe(operators_1.mergeMap(function (aToolsOpts) { return _updateVisualStudioCodeLaunch(aSrcDir, name, aToolsOpts, logger); }));
            var vcTasks = toolsOpts.pipe(operators_1.mergeMap(function (aToolsOpts) { return _updateVisualStudioCodeTasks(aSrcDir, name, aToolsOpts, logger); }));
            var appModule = _rewriteAppModule(aSrcDir, name, logger);
            var routingModule = _rewriteRoutingModule(aSrcDir, name, logger);
            var environment = _rewriteEnvironment(aSrcDir, name, apiUrl, logger);
            var index = _rewriteIndex(aSrcDir, name, logger);
            var lint = _rewriteLint(aSrcDir, name, logger);
            var ngCli = _rewriteCli(aSrcDir, name, logger);
            var manifest = _addManifest(aSrcDir, name, _this.options);
            // files that can be run in parallel
            var rxFiles = rxjs_1.merge(typings, vcLaunch, vcTasks, vcSettings, wchToolsOptions, appConfig);
            // merge all of these
            return rxjs_1.merge(app, rxjs_1.concat(data, layouts, components, pkg, rxFiles, appModule, routingModule, environment, index, lint, manifest, ngCli, copyResources));
        }), operators_1.distinct(), operators_1.finalize(function () { return process_1.chdir(currentDir); }));
    };
    CreateProjectCommand.TYPE = 'project';
    CreateProjectCommand.COMMAND = constants_1.CMD_CREATE;
    return CreateProjectCommand;
}(abstract_command_1.AbstractCommand));
exports.CreateProjectCommand = CreateProjectCommand;
