"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
/* Copyright IBM Corp. 2017 */
var lodash_1 = require("lodash");
var path_1 = require("path");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var uuid_1 = require("uuid");
var file_utils_1 = require("../utils/file.utils");
var constants_1 = require("./../constants");
var options_1 = require("./../options");
var json_1 = require("./../utils/json");
var revision_utils_1 = require("./../utils/revision.utils");
var rx_copy_1 = require("./../utils/rx.copy");
var rx_walk_1 = require("./../utils/rx.walk");
var abstract_command_1 = require("./abstract.command");
exports.UUID_MANIFEST_TYPE = '3900cd47-0f9a-45a9-892f-3f9a3e42c751';
exports.UUID_MANIFEST_LAYOUT = '72b59cc2-3664-4a55-a8df-3604b503ed47';
exports.UUID_FAVICONS_LAYOUT = '3fb75fda-4ff2-42a0-be53-69837df22f43';
exports.KEY_MANIFEST_ID = 'manifestId';
var CONTENT_REGEXP = /content.*_cmd\.json/g;
function _isContent(aFileName) {
    return CONTENT_REGEXP.test(aFileName);
}
function _getLayoutMappingForType(aTypeId, aTools) {
    var foundId = Object.keys(aTools.layoutMappings).find(function (id) { return aTools.layoutMappings[id].type.id === aTypeId; });
    return foundId ? aTools.layoutMappings[foundId] : undefined;
}
function _findManifestLayout(aTools, aLogger) {
    // locate
    return lodash_1.find(aTools.layouts, function (layout) { return layout.id === exports.UUID_MANIFEST_LAYOUT; });
}
function _findFaviconsLayout(aTools, aLogger) {
    // locate
    return lodash_1.find(aTools.layouts, function (layout) { return layout.id === exports.UUID_FAVICONS_LAYOUT; });
}
function _createLayoutMapping(aTools, aLogger) {
    // new ID
    var id = uuid_1.v4();
    // load the template
    var srcFile = path_1.join(__dirname, '..', '..', '..', 'assets', 'manifest.layout.mapping.json');
    var dstFolder = path_1.join(aTools.root, 'layout-mappings');
    var dstFile = path_1.join(dstFolder, id + ".json");
    // create the folder
    var rxDstFolder = rx_walk_1.rxMkdirp(dstFolder);
    // log this
    aLogger.next("Creating layout mapping with ID " + id + " ...");
    // load the file
    var rxMapping = rx_walk_1.readFile(srcFile).pipe(operators_1.map(function (data) { return JSON.parse(data); }), operators_1.map(function (mapping) {
        // update
        mapping.id = id;
        mapping.name = id;
        mapping.type = {
            id: exports.UUID_MANIFEST_TYPE
        };
        // ok
        var newMapping = json_1.canonicalizeJSON(revision_utils_1.addRevisionId(mapping));
        aTools.layoutMappings[id] = newMapping;
        aTools.idToFile[id] = dstFile;
        // ok
        return newMapping;
    }));
    // combine
    return rxjs_1.combineLatest(rxMapping, rxDstFolder).pipe(operators_1.mergeMap(function (_a) {
        var mapping = _a[0];
        return rx_walk_1.writeFile(dstFile, JSON.stringify(mapping, undefined, 2));
    }), operators_1.shareReplay());
}
function _addLayoutToMappings(aLayoutID, aMappings, aTools) {
    // add to existing layouts
    var layouts = aMappings.layouts || (aMappings.layouts = []);
    var exLayout = layouts.find(function (m) { return m.id === aLayoutID; });
    // add the layout if it does not exist
    if (!exLayout) {
        layouts.push({
            id: aLayoutID
        });
    }
    // add the default
    var defLayout = aMappings.defaultLayout;
    if (!defLayout) {
        aMappings.defaultLayout = {
            id: aLayoutID
        };
    }
}
function _addLayoutToMapping(aLayoutID, aMapping, aTools, aLogger) {
    // log this
    aLogger.next("_addLayoutToMapping [" + aLayoutID + "] ...");
    // find the layout
    var mappings = aMapping.mappings || (aMapping.mappings = []);
    // iterate
    mappings.forEach(function (m) { return _addLayoutToMappings(aLayoutID, m, aTools); });
    // handle the empty case
    if (mappings.length === 0) {
        var m = {};
        _addLayoutToMappings(aLayoutID, m, aTools);
        mappings.push(m);
    }
    // write the beast
    var fileName = aTools.idToFile[aMapping.id];
    // update the mappings
    aTools.layoutMappings[aMapping.id] = aMapping;
    // write
    return rx_walk_1.writeFile(fileName, JSON.stringify(aMapping, undefined, 2));
}
function _fixSlashes(aPath) {
    // replace the slashes
    var p = aPath.replace(/\\/g, '/');
    if (!p.startsWith('/')) {
        p = '/' + p;
    }
    // ok
    return p;
}
function _writeLayoutAndMapping(aRoot, aTypeId, aLayoutId, aTools, aLogger) {
    // init
    return rxjs_1.of(aLayoutId).pipe(operators_1.mergeMap(function (id) {
        // find the layout mapping
        var exMapping = _getLayoutMappingForType(aTypeId, aTools);
        // resolve
        var mappingFile = exMapping ? rxjs_1.empty() : _createLayoutMapping(aTools, aLogger);
        var mapping = exMapping ? rxjs_1.of(exMapping) :
            mappingFile.pipe(operators_1.mergeMap(function (file) { return rx_walk_1.readFile(file); }), operators_1.map(function (data) { return JSON.parse(data); }));
        // update the mapping file with the layout
        var newMapping = mapping.pipe(operators_1.mergeMap(function (m) { return _addLayoutToMapping(id, m, aTools, aLogger); }));
        // combine all
        return rxjs_1.merge(mappingFile, newMapping).pipe(operators_1.distinct());
    }));
}
function _writeContentItem(aName, aRoot, aDataDir, aState) {
    // id
    var id = uuid_1.v4();
    aState.contentItemId = id;
    // construct the target folder
    var targetFolder = rx_walk_1.rxMkdirp(path_1.join(aDataDir, 'content'));
    // load the content item
    var srcFile = path_1.join(__dirname, '..', '..', '..', 'assets', 'manifest.content.json');
    var fileName = id + "_cmd.json";
    // name
    var name = lodash_1.camelCase(aName + " Web App Manifest");
    // read
    return rx_walk_1.readFile(srcFile).pipe(operators_1.map(function (data) { return JSON.parse(data); }), operators_1.map(function (data) {
        // add the name
        data.id = id;
        data.name = name;
        // update the description
        data.elements.short_name.value = aName;
        data.elements.name.value = aName;
        // context root
        data.elements.context_root.value = aRoot;
        // inject the revision field
        return json_1.canonicalizeJSON(revision_utils_1.addRevisionId(data));
    }), operators_1.mergeMap(function (data) { return targetFolder.pipe(operators_1.mergeMap(function (tgt) { return rx_walk_1.writeFile(path_1.join(tgt, fileName), JSON.stringify(data, undefined, 2)); })); }));
}
var HEAD_REGEXP = /<\/head>/g;
var MANIFEST_REGEXP = /<link\s+[^>]*rel\s*=\s*\"manifest\"[^>]*>/g;
var FAVICON_REGEXP = /<link\s+[^>]*rel\s*=\s*\"icon\"[^>]*>/g;
function _fixIndex(aIndexContent) {
    // the source
    var src = aIndexContent;
    // rewrite the favicon
    if (FAVICON_REGEXP.test(src)) {
    }
    else {
        // locate the end of the header and add the link
    }
    // rewrite the metadata
    /*    const manifest = `<link rel="manifest" href="${encodeURIComponent(MANIFEST_JSON)}">`;
        if (MANIFEST_REGEXP.test(aIndexContent)) {
            // replace
            src = src.replace(MANIFEST_REGEXP, manifest);
        } else {
            // locate the end of the header and add the links
            src = src.replace(HEAD_REGEXP, `${manifest}\n</head>`);
        } */
    // ok
    return src;
}
function _rewriteIndex(aSrcDir, aName, aLogger) {
    // name
    var name = path_1.join(aSrcDir, 'index.html');
    return rx_walk_1.readFile(name).pipe(operators_1.map(function (idx) { return _fixIndex(idx); }), operators_1.mergeMap(function (data) { return rx_walk_1.writeFile(name, data); }));
}
// just copy the resources
function _copyResources(aDataDir) {
    // source
    var srcDir = path_1.join(__dirname, '..', '..', '..', 'assets', 'manifest');
    var dstDir = aDataDir;
    // copy
    return rx_copy_1.rxCopyDir(srcDir, dstDir);
}
/**
 * Copies the resources
 *
 * @param aName     name
 * @param aDataDir  data directory
 */
function _copyManifest(aName, aRoot, aDataDir, aState) {
    // copy the files
    var files = _copyResources(aDataDir);
    // rewritten
    var content = _writeContentItem(aName, aRoot, aDataDir, aState);
    // combine
    return rxjs_1.merge(files, content).pipe(operators_1.distinct());
}
function _rewritePackage(aSrcDir, aState) {
    // load the package json
    var filename = path_1.join(aSrcDir, 'package.json');
    // load
    return rx_walk_1.readFile(filename).pipe(operators_1.map(function (data) { return JSON.parse(data); }), operators_1.tap(function (data) { return data.config[exports.KEY_MANIFEST_ID] = aState.contentItemId; }), operators_1.map(function (data) { return JSON.stringify(data, undefined, 2); }), operators_1.mergeMap(function (data) { return rx_walk_1.writeFile(filename, data); }));
}
function _handleLayoutAndMapping(aRoot, aWchTools, aLogger) {
    // construct the mappings
    return rxjs_1.concat(_writeLayoutAndMapping(aRoot, exports.UUID_MANIFEST_TYPE, exports.UUID_MANIFEST_LAYOUT, aWchTools, aLogger), _writeLayoutAndMapping(aRoot, exports.UUID_MANIFEST_TYPE, exports.UUID_FAVICONS_LAYOUT, aWchTools, aLogger));
}
/**
 * Generates angular components based on type and layout mapping definitions
 *
 * create webappmanifest --data DATA_FOLDER
 */
var CreateWebAppManifestCommand = /** @class */ (function (_super) {
    __extends(CreateWebAppManifestCommand, _super);
    function CreateWebAppManifestCommand(options) {
        return _super.call(this, options) || this;
    }
    CreateWebAppManifestCommand.prototype.process = function () {
        // logger
        var logger = this.getLogger();
        // data directory
        var opts = this.getOptions();
        var dataDirOpt = this.getDataDir();
        var rxDataDir = this.rxMkdirp(dataDirOpt);
        // the directories
        var srcDirOpt = options_1.optNormalizeDir(file_utils_1.firstOf(opts.src) || '.');
        var rxSrcDir = this.rxMkdirp(srcDirOpt);
        // root
        var root = opts.root || '';
        // relative or absolute
        var bRelative = !!opts.relative;
        // log the settings
        logger.next("Source dir [" + srcDirOpt + "], relative [" + bRelative + "].");
        // name
        var name = opts.name || 'Application';
        // state
        var state = {};
        // tools
        var rxWchTools = this.getWchTools();
        // manifest
        var rxManifest = rxjs_1.concat(rxDataDir.pipe(operators_1.mergeMap(function (aDataDir) { return _copyManifest(name, root, aDataDir, state); })), rxSrcDir.pipe(operators_1.mergeMap(function (aSourceDir) { return _rewritePackage(aSourceDir, state); })));
        // layouts
        var rxLayouts = rxWchTools.pipe(operators_1.mergeMap(function (aTools) { return _handleLayoutAndMapping(root, aTools, logger); }));
        // combine
        return rxjs_1.merge(rxManifest, rxLayouts);
    };
    CreateWebAppManifestCommand.TYPE = 'webappmanifest';
    CreateWebAppManifestCommand.COMMAND = constants_1.CMD_CREATE;
    return CreateWebAppManifestCommand;
}(abstract_command_1.AbstractCommand));
exports.CreateWebAppManifestCommand = CreateWebAppManifestCommand;
