import { UnaryFunction } from 'rxjs';

export type OutboundEdges<T> = UnaryFunction<T, T[]>;

function _visitNodes<T>(aNodes: T[], aEdges: OutboundEdges<T>, aSet: Set<T>) {
    // iterate over all nodes
    if (aNodes && (aNodes.length > 0)) {
        // iterate
        aNodes.forEach(node => {
            // check if we know the node
            if (!aSet.has(node)) {
                // add the node
                aSet.add(node);
                // recurse
                _visitNodes(aEdges(node), aEdges, aSet);
            }
        });
    }
}

export function connectedComponents<T>(aNodes: T[], aEdges: OutboundEdges<T>): T[] {
    // set of visited nodes
    const result = new Set<T>();
    // recurse
    _visitNodes(aNodes, aEdges, result);
    // convert
    return Array.from(result.keys());
}