"use strict";
exports.__esModule = true;
var path_1 = require("path");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var options_1 = require("../options");
var rx_walk_1 = require("./rx.walk");
function _rxGetContentDependencies(aContentId, aTools, aCycle) {
    // check if we have a type
    if (aContentId) {
        // check
        if (!aCycle[aContentId]) {
            // add the dependency
            aCycle[aContentId] = aContentId;
            // asset file
            var contentFile = aTools.idToFile[aContentId];
            // get the type
            var content_1 = aTools.content[aContentId];
            // check
            if (contentFile && content_1) {
                // add the asset file
                var res_1 = [];
                res_1.push(rxjs_1.of(contentFile));
                // iterate over the elements
                if (content_1.elements) {
                    // iterate
                    Object.keys(content_1.elements).forEach(function (key) {
                        var el = content_1.elements[key];
                        if (el.elementType === 'reference') {
                            // check values
                            if (el.values) {
                                el.values.forEach(function (v) { return res_1.push(_rxGetContentDependencies(v, aTools, aCycle)); });
                            }
                            if (el.value) {
                                res_1.push(_rxGetContentDependencies(el.value, aTools, aCycle));
                            }
                        }
                        else if ((el.elementType === 'image') && el.asset && el.asset.id) {
                            // copy the image
                            res_1.push(_rxGetAssetDependencies(el.asset.id, aTools, aCycle));
                        }
                    });
                }
                // thumb
                if (content_1.thumbnail && content_1.thumbnail.id) {
                    res_1.push(_rxGetAssetDependencies(content_1.thumbnail.id, aTools, aCycle));
                }
                // type
                if (content_1.typeId) {
                    res_1.push(_rxGetTypeDependencies(content_1.typeId, aTools, aCycle));
                }
                // merge
                return rxjs_1.merge.apply(void 0, res_1);
            }
            // nothing
            return rxjs_1.empty();
        }
    }
    // nothing
    return rxjs_1.empty();
}
function _rxGetAssetDependencies(aAssetId, aTools, aCycle) {
    // check if we have a type
    if (aAssetId) {
        // check
        if (!aCycle[aAssetId]) {
            // add the dependency
            aCycle[aAssetId] = aAssetId;
            // handle the assets
            return aTools.rxAssets.pipe(operators_1.mergeMap(function (assets) {
                // asset file
                var assetFile = aTools.idToFile[aAssetId];
                // get the type
                var asset = assets[aAssetId];
                // check
                if (assetFile && asset) {
                    // add the asset file
                    var res = [];
                    res.push(rxjs_1.of(assetFile));
                    // resolve the asset
                    var path = path_1.parse(assetFile);
                    // append the binary asset
                    var assetBinaryFile = path_1.join(path.dir, asset.fileName);
                    res.push(rxjs_1.of(assetBinaryFile));
                    // append the renditions
                    res.push(aTools.rxRenditions.pipe(operators_1.map(function (renditions) { return Object.keys(renditions).
                        filter(function (key) { return renditions[key].asset.id === aAssetId; }).
                        map(function (key) { return aTools.idToFile[key]; }); }), operators_1.mergeMap(function (files) { return rxjs_1.from(files); })));
                    // merge
                    return rxjs_1.merge.apply(void 0, res);
                }
                // nothing
                return rxjs_1.empty();
            }));
        }
    }
    // nothing
    return rxjs_1.empty();
}
function _rxGetTypeDependencies(aTypeId, aTools, aCycle) {
    // check if we have a type
    if (aTypeId) {
        // check
        if (!aCycle[aTypeId]) {
            // add the dependency
            aCycle[aTypeId] = aTypeId;
            // the type itself
            var typeFile = aTools.idToFile[aTypeId];
            // get the type
            var type = aTools.types[aTypeId];
            // handle
            if (typeFile && type) {
                // handle dependencies
                var res = [];
                res.push(rxjs_1.of(typeFile));
                // thumbnail
                var thumb = type['thumbnail'];
                if (thumb) {
                    res.push(_rxGetAssetDependencies(thumb, aTools, aCycle));
                }
                // ok
                return rxjs_1.merge.apply(void 0, res);
            }
        }
    }
    // nothing
    return rxjs_1.empty();
}
function rxGetTypeDependencies(aTypeId, aTools) {
    return _rxGetTypeDependencies(aTypeId, aTools, {});
}
exports.rxGetTypeDependencies = rxGetTypeDependencies;
function rxGetContentDependencies(aMatcher, aTools) {
    // make sure the root ends with a slahs
    var prefix = options_1.optNormalizeDir(aTools.root);
    // cycle
    var cycle = {};
    // iterate over all content items
    var rxTotal = rxjs_1.merge.apply(void 0, Object.keys(aTools.content).filter(function (key) { return aMatcher(aTools.content[key].name); }).map(function (key) { return _rxGetContentDependencies(key, aTools, cycle); })).pipe(operators_1.filter(function (file) { return file.startsWith(prefix); }), operators_1.distinct(), operators_1.mergeMap(function (file) { return rx_walk_1.rxGetFileDescriptor(file, prefix); }));
    // ok
    return rxTotal;
}
exports.rxGetContentDependencies = rxGetContentDependencies;
