"use strict";
exports.__esModule = true;
var fs_extra_1 = require("fs-extra");
var graceful_fs_1 = require("graceful-fs");
var rimraf = require("rimraf");
var rxjs_1 = require("rxjs");
var verror_1 = require("verror");
var opt = {
    unlink: graceful_fs_1.unlink,
    chmod: graceful_fs_1.chmod,
    stat: graceful_fs_1.stat,
    lstat: graceful_fs_1.lstat,
    rmdir: graceful_fs_1.rmdir,
    readdir: graceful_fs_1.readdir
};
function _rxDeleteFile(aPath) {
    return rxjs_1.Observable.create(function (o) {
        // dispatch
        graceful_fs_1.unlink(aPath, function (rimRafErr) {
            // test for error
            if (rimRafErr) {
                o.error(new verror_1.VError(rimRafErr, 'Unable to delete file [%s].', aPath));
            }
            else {
                o.next(aPath);
                o.complete();
            }
        });
    });
}
exports.rxDeleteFile = _rxDeleteFile;
function _rxDeleteDir(aPath) {
    return rxjs_1.Observable.create(function (o) {
        // dispatch
        rimraf(aPath, opt, function (rimRafErr) {
            // test for error
            if (rimRafErr) {
                o.error(new verror_1.VError(rimRafErr, 'Unable to delete directory [%s].', aPath));
            }
            else {
                o.next(aPath);
                o.complete();
            }
        });
    });
}
exports.rxDeleteDir = _rxDeleteDir;
function _rxEmptyDir(aPath) {
    return rxjs_1.Observable.create(function (o) {
        // dispatch
        fs_extra_1.emptyDir(aPath, function (emptyError) {
            // test for error
            if (emptyError) {
                o.error(new verror_1.VError(emptyError, 'Unable to empty directory [%s].', aPath));
            }
            else {
                o.next(aPath);
                o.complete();
            }
        });
    });
}
exports.rxEmptyDir = _rxEmptyDir;
