"use strict";
exports.__esModule = true;
var lodash_1 = require("lodash");
var npa = require("npm-package-arg");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var semver_1 = require("semver");
var url_1 = require("url");
var constants_1 = require("../constants");
var assert_1 = require("./assert");
var rx_download_1 = require("./rx.download");
function getMatchingVersion(aVersions, aRange) {
    // find all matches
    return semver_1.maxSatisfying(aVersions, aRange);
}
exports.getMatchingVersion = getMatchingVersion;
function parseReference(aReference) {
    // parse
    var res = npa(aReference);
    // sanity check
    assert_1.assertParameter(res.name, constants_1.OPTION_NAME);
    assert_1.assertParameter(res.escapedName, constants_1.OPTION_NAME);
    // get the spec
    var spec = res.rawSpec ? (res.rawSpec === 'latest') ? '*' : res.rawSpec : '*';
    // default
    return {
        name: res.name,
        spec: spec,
        escapedName: res.escapedName
    };
}
exports.parseReference = parseReference;
function _getPackageJsonFromResponse(aResp, aBaseUrl, aReference) {
    // read the versions
    var versions = Object.keys(aResp.versions);
    // select the best one
    var matching = getMatchingVersion(versions, aReference.spec);
    // now return the package
    var version = aResp.versions[matching];
    if (lodash_1.isString(version)) {
        // make an extra request
        return rx_download_1.rxGetJson(aBaseUrl + '/' + encodeURIComponent(matching));
    }
    // default
    return rxjs_1.of(version);
}
function getPackageJson(aBaseUrl, aReference) {
    // the url
    var url = url_1.resolve(aBaseUrl, aReference.escapedName);
    // make the request
    return rx_download_1.rxGetJson(url).pipe(operators_1.mergeMap(function (resp) { return _getPackageJsonFromResponse(resp, url, aReference); }));
}
exports.getPackageJson = getPackageJson;
